@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -31,10 +31,15 @@
 #include <VBox/sup.h>
 #include <VBox/log.h>
 #include <iprt/stdarg.h>
+#include <iprt/thread.h>
 
 RT_C_DECLS_BEGIN
 
-/** @defgroup grp_vmm       The Virtual Machine Monitor API
+/** @defgroup grp_vmm       The Virtual Machine Monitor
+ * @{
+ */
+
+/** @defgroup grp_vmm_api   The Virtual Machine Monitor API
  * @{
  */
 
@@ -63,6 +68,10 @@ typedef enum VMMSWITCHER
     VMMSWITCHER_AMD64_TO_PAE,
     /** Switcher for AMD64 host paging to AMD64 shadow paging. */
     VMMSWITCHER_AMD64_TO_AMD64,
+    /** Stub switcher for 32-bit and PAE. */
+    VMMSWITCHER_X86_STUB,
+    /** Stub switcher for AMD64. */
+    VMMSWITCHER_AMD64_STUB,
     /** Used to make a count for array declarations and suchlike. */
     VMMSWITCHER_MAX,
     /** The usual 32-bit paranoia. */
@@ -81,6 +90,10 @@ typedef enum VMMCALLRING3
     VMMCALLRING3_PDM_LOCK,
     /** Acquire the critical section specified as argument.  */
     VMMCALLRING3_PDM_CRIT_SECT_ENTER,
+    /** Enter the R/W critical section (in argument) exclusively.  */
+    VMMCALLRING3_PDM_CRIT_SECT_RW_ENTER_EXCL,
+    /** Enter the R/W critical section (in argument) shared.  */
+    VMMCALLRING3_PDM_CRIT_SECT_RW_ENTER_SHARED,
     /** Acquire the PGM lock. */
     VMMCALLRING3_PGM_LOCK,
     /** Grow the PGM shadow page pool. */
@@ -103,7 +116,8 @@ typedef enum VMMCALLRING3
     VMMCALLRING3_VM_SET_RUNTIME_ERROR,
     /** Signal a ring 0 assertion. */
     VMMCALLRING3_VM_R0_ASSERTION,
-    /** Ring switch to force preemption. */
+    /** Ring switch to force preemption.  This is also used by PDMCritSect to
+     *  handle VERR_INTERRUPTED in kernel context. */
     VMMCALLRING3_VM_R0_PREEMPT,
     /** Sync the FTM state with the standby node. */
     VMMCALLRING3_FTM_SET_CHECKPOINT,
@@ -112,17 +126,16 @@ typedef enum VMMCALLRING3
 } VMMCALLRING3;
 
 /**
- * VMMR3AtomicExecuteHandler callback function.
+ * VMMRZCallRing3 notification callback.
  *
  * @returns VBox status code.
- * @param   pVM     Pointer to the shared VM structure.
- * @param   pvUser  User specified argument
- *
- * @todo missing prefix.
+ * @param   pVCpu           The cross context virtual CPU structure.
+ * @param   enmOperation    The operation causing the ring-3 jump.
+ * @param   pvUser          The user argument.
  */
-typedef DECLCALLBACK(int) FNATOMICHANDLER(PVM pVM, void *pvUser);
-/** Pointer to a FNMMATOMICHANDLER(). */
-typedef FNATOMICHANDLER *PFNATOMICHANDLER;
+typedef DECLCALLBACK(int) FNVMMR0CALLRING3NOTIFICATION(PVMCPU pVCpu, VMMCALLRING3 enmOperation, void *pvUser);
+/** Pointer to a FNRTMPNOTIFICATION(). */
+typedef FNVMMR0CALLRING3NOTIFICATION *PFNVMMR0CALLRING3NOTIFICATION;
 
 /**
  * Rendezvous callback.
@@ -131,9 +144,9 @@ typedef FNATOMICHANDLER *PFNATOMICHANDLER;
  *          informational status code other than the ones used by EM for
  *          scheduling.
  *
- * @param   pVM         The VM handle.
- * @param   pVCpu       The handle of the calling virtual CPU.
- * @param   pvUser      The user argument.
+ * @param   pVM     The cross context VM structure.
+ * @param   pVCpu   The cross context virtual CPU structure of the calling EMT.
+ * @param   pvUser  The user argument.
  */
 typedef DECLCALLBACK(VBOXSTRICTRC) FNVMMEMTRENDEZVOUS(PVM pVM, PVMCPU pVCpu, void *pvUser);
 /** Pointer to a rendezvous callback function. */
@@ -233,83 +246,100 @@ typedef struct VMM2USERMETHODS
 #define VMM2USERMETHODS_VERSION       UINT32_C(0x00020001)
 
 
-VMMDECL(RTRCPTR)     VMMGetStackRC(PVMCPU pVCpu);
-VMMDECL(VMCPUID)     VMMGetCpuId(PVM pVM);
-VMMDECL(PVMCPU)      VMMGetCpu(PVM pVM);
-VMMDECL(PVMCPU)      VMMGetCpu0(PVM pVM);
-VMMDECL(PVMCPU)      VMMGetCpuById(PVM pVM, VMCPUID idCpu);
-VMMDECL(uint32_t)    VMMGetSvnRev(void);
-VMMDECL(VMMSWITCHER) VMMGetSwitcher(PVM pVM);
-VMMDECL(void)        VMMTrashVolatileXMMRegs(void);
-
-/** @def VMMIsHwVirtExtForced
- * Checks if forced to use the hardware assisted virtualization extensions.
+/**
+ * Checks whether we've armed the ring-0 long jump machinery.
  *
- * This is intended for making setup decisions where we can save resources when
- * using hardware assisted virtualization.
- *
- * @returns true / false.
- * @param   pVM     Pointer to the shared VM structure.
+ * @returns @c true / @c false
+ * @param   a_pVCpu     The caller's cross context virtual CPU structure.
+ * @thread  EMT
+ * @sa      VMMR0IsLongJumpArmed
  */
-#define VMMIsHwVirtExtForced(pVM)   ((pVM)->fHwVirtExtForced)
+#ifdef IN_RING0
+# define VMMIsLongJumpArmed(a_pVCpu)                VMMR0IsLongJumpArmed(a_pVCpu)
+#else
+# define VMMIsLongJumpArmed(a_pVCpu)                (false)
+#endif
 
 
-#ifdef IN_RING3
-/** @defgroup grp_vmm_r3    The VMM Host Context Ring 3 API
- * @ingroup grp_vmm
+VMM_INT_DECL(RTRCPTR)       VMMGetStackRC(PVMCPU pVCpu);
+VMMDECL(VMCPUID)            VMMGetCpuId(PVM pVM);
+VMMDECL(PVMCPU)             VMMGetCpu(PVM pVM);
+VMMDECL(PVMCPU)             VMMGetCpu0(PVM pVM);
+VMMDECL(PVMCPU)             VMMGetCpuById(PVM pVM, VMCPUID idCpu);
+VMMR3DECL(PVMCPU)           VMMR3GetCpuByIdU(PUVM pVM, VMCPUID idCpu);
+VMM_INT_DECL(uint32_t)      VMMGetSvnRev(void);
+VMM_INT_DECL(VMMSWITCHER)   VMMGetSwitcher(PVM pVM);
+VMM_INT_DECL(bool)          VMMIsInRing3Call(PVMCPU pVCpu);
+VMM_INT_DECL(void)          VMMTrashVolatileXMMRegs(void);
+VMM_INT_DECL(int)           VMMPatchHypercall(PVM pVM, void *pvBuf, size_t cbBuf, size_t *pcbWritten);
+VMM_INT_DECL(void)          VMMHypercallsEnable(PVMCPU pVCpu);
+VMM_INT_DECL(void)          VMMHypercallsDisable(PVMCPU pVCpu);
+
+
+#if defined(IN_RING3) || defined(DOXYGEN_RUNNING)
+/** @defgroup grp_vmm_api_r3    The VMM Host Context Ring 3 API
  * @{
  */
 VMMR3_INT_DECL(int)     VMMR3Init(PVM pVM);
 VMMR3_INT_DECL(int)     VMMR3InitR0(PVM pVM);
+# ifdef VBOX_WITH_RAW_MODE
 VMMR3_INT_DECL(int)     VMMR3InitRC(PVM pVM);
+# endif
 VMMR3_INT_DECL(int)     VMMR3InitCompleted(PVM pVM, VMINITCOMPLETED enmWhat);
 VMMR3_INT_DECL(int)     VMMR3Term(PVM pVM);
 VMMR3_INT_DECL(void)    VMMR3Relocate(PVM pVM, RTGCINTPTR offDelta);
 VMMR3_INT_DECL(int)     VMMR3UpdateLoggers(PVM pVM);
 VMMR3DECL(const char *) VMMR3GetRZAssertMsg1(PVM pVM);
 VMMR3DECL(const char *) VMMR3GetRZAssertMsg2(PVM pVM);
-VMMR3_INT_DECL(int)     VMMR3GetImportRC(PVM pVM, const char *pszSymbol, PRTRCPTR pRCPtrValue);
 VMMR3_INT_DECL(int)     VMMR3SelectSwitcher(PVM pVM, VMMSWITCHER enmSwitcher);
-VMMR3_INT_DECL(int)     VMMR3DisableSwitcher(PVM pVM);
 VMMR3_INT_DECL(RTR0PTR) VMMR3GetHostToGuestSwitcher(PVM pVM, VMMSWITCHER enmSwitcher);
+VMMR3_INT_DECL(int)     VMMR3HmRunGC(PVM pVM, PVMCPU pVCpu);
+# ifdef VBOX_WITH_RAW_MODE
 VMMR3_INT_DECL(int)     VMMR3RawRunGC(PVM pVM, PVMCPU pVCpu);
-VMMR3_INT_DECL(int)     VMMR3HwAccRunGC(PVM pVM, PVMCPU pVCpu);
+VMMR3DECL(int)          VMMR3ResumeHyper(PVM pVM, PVMCPU pVCpu);
+VMMR3_INT_DECL(int)     VMMR3GetImportRC(PVM pVM, const char *pszSymbol, PRTRCPTR pRCPtrValue);
 VMMR3DECL(int)          VMMR3CallRC(PVM pVM, RTRCPTR RCPtrEntry, unsigned cArgs, ...);
 VMMR3DECL(int)          VMMR3CallRCV(PVM pVM, RTRCPTR RCPtrEntry, unsigned cArgs, va_list args);
+# endif
 VMMR3DECL(int)          VMMR3CallR0(PVM pVM, uint32_t uOperation, uint64_t u64Arg, PSUPVMMR0REQHDR pReqHdr);
-VMMR3DECL(int)          VMMR3ResumeHyper(PVM pVM, PVMCPU pVCpu);
 VMMR3DECL(void)         VMMR3FatalDump(PVM pVM, PVMCPU pVCpu, int rcErr);
 VMMR3_INT_DECL(void)    VMMR3YieldSuspend(PVM pVM);
 VMMR3_INT_DECL(void)    VMMR3YieldStop(PVM pVM);
 VMMR3_INT_DECL(void)    VMMR3YieldResume(PVM pVM);
-VMMR3_INT_DECL(void)    VMMR3SendSipi(PVM pVM, VMCPUID idCpu, uint32_t uVector);
+VMMR3_INT_DECL(void)    VMMR3SendStartupIpi(PVM pVM, VMCPUID idCpu, uint32_t uVector);
 VMMR3_INT_DECL(void)    VMMR3SendInitIpi(PVM pVM, VMCPUID idCpu);
 VMMR3DECL(int)          VMMR3RegisterPatchMemory(PVM pVM, RTGCPTR pPatchMem, unsigned cbPatchMem);
 VMMR3DECL(int)          VMMR3DeregisterPatchMemory(PVM pVM, RTGCPTR pPatchMem, unsigned cbPatchMem);
 VMMR3DECL(int)          VMMR3EmtRendezvous(PVM pVM, uint32_t fFlags, PFNVMMEMTRENDEZVOUS pfnRendezvous, void *pvUser);
-VMMR3_INT_DECL(bool)    VMMR3EmtRendezvousSetDisabled(PVMCPU pVCpu, bool fDisabled);
 /** @defgroup grp_VMMR3EmtRendezvous_fFlags     VMMR3EmtRendezvous flags
  *  @{ */
 /** Execution type mask. */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_MASK            UINT32_C(0x00000007)
 /** Invalid execution type. */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_INVALID         UINT32_C(0)
-/** Let the EMTs execute the callback one by one (in no particular order). */
+/** Let the EMTs execute the callback one by one (in no particular order).
+ * Recursion from within the callback possible.  */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_ONE_BY_ONE      UINT32_C(1)
-/** Let all the EMTs execute the callback at the same time. */
+/** Let all the EMTs execute the callback at the same time.
+ * Cannot recurse from the callback.  */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_ALL_AT_ONCE     UINT32_C(2)
-/** Only execute the callback on one EMT (no particular one). */
+/** Only execute the callback on one EMT (no particular one).
+ * Recursion from within the callback possible.  */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_ONCE            UINT32_C(3)
-/** Let the EMTs execute the callback one by one in ascending order. */
+/** Let the EMTs execute the callback one by one in ascending order.
+ * Recursion from within the callback possible. */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_ASCENDING       UINT32_C(4)
-/** Let the EMTs execute the callback one by one in descending order. */
+/** Let the EMTs execute the callback one by one in descending order.
+ * Recursion from within the callback possible. */
 #define VMMEMTRENDEZVOUS_FLAGS_TYPE_DESCENDING      UINT32_C(5)
 /** Stop after the first error.
  * This is not valid for any execution type where more than one EMT is active
  * at a time. */
 #define VMMEMTRENDEZVOUS_FLAGS_STOP_ON_ERROR        UINT32_C(0x00000008)
+/** Use VMREQFLAGS_PRIORITY when contacting the EMTs. */
+#define VMMEMTRENDEZVOUS_FLAGS_PRIORITY             UINT32_C(0x00000010)
 /** The valid flags. */
-#define VMMEMTRENDEZVOUS_FLAGS_VALID_MASK           UINT32_C(0x0000000f)
+#define VMMEMTRENDEZVOUS_FLAGS_VALID_MASK           UINT32_C(0x0000001f)
 /** @} */
 VMMR3_INT_DECL(int)     VMMR3EmtRendezvousFF(PVM pVM, PVMCPU pVCpu);
 VMMR3_INT_DECL(int)     VMMR3ReadR0Stack(PVM pVM, VMCPUID idCpu, RTHCUINTPTR R0Addr, void *pvBuf, size_t cbRead);
@@ -317,8 +347,7 @@ VMMR3_INT_DECL(int)     VMMR3ReadR0Stack(PVM pVM, VMCPUID idCpu, RTHCUINTPTR R0A
 #endif /* IN_RING3 */
 
 
-/** @defgroup grp_vmm_r0    The VMM Host Context Ring 0 API
- * @ingroup grp_vmm
+/** @defgroup grp_vmm_api_r0    The VMM Host Context Ring 0 API
  * @{
  */
 
@@ -330,7 +359,7 @@ typedef enum VMMR0OPERATION
     /** Run guest context. */
     VMMR0_DO_RAW_RUN = SUP_VMMR0_DO_RAW_RUN,
     /** Run guest code using the available hardware acceleration technology. */
-    VMMR0_DO_HWACC_RUN = SUP_VMMR0_DO_HWACC_RUN,
+    VMMR0_DO_HM_RUN = SUP_VMMR0_DO_HM_RUN,
     /** Official NOP that we use for profiling. */
     VMMR0_DO_NOP = SUP_VMMR0_DO_NOP,
     /** Official slow iocl NOP that we use for profiling. */
@@ -362,9 +391,9 @@ typedef enum VMMR0OPERATION
     /** Call VMMR0 Per VM Termination. */
     VMMR0_DO_VMMR0_TERM,
     /** Setup the hardware accelerated raw-mode session. */
-    VMMR0_DO_HWACC_SETUP_VM,
+    VMMR0_DO_HM_SETUP_VM,
     /** Attempt to enable or disable hardware accelerated raw-mode. */
-    VMMR0_DO_HWACC_ENABLE,
+    VMMR0_DO_HM_ENABLE,
     /** Calls function in the hypervisor.
      * The caller must setup the hypervisor context so the call will be performed.
      * The difference between VMMR0_DO_RUN_GC and this one is the handling of
@@ -485,36 +514,44 @@ typedef struct GCFGMVALUEREQ
  */
 typedef GCFGMVALUEREQ *PGCFGMVALUEREQ;
 
-VMMR0DECL(int)      VMMR0EntryInt(PVM pVM, VMMR0OPERATION enmOperation, void *pvArg);
-VMMR0DECL(void)     VMMR0EntryFast(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation);
-VMMR0DECL(int)      VMMR0EntryEx(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation, PSUPVMMR0REQHDR pReq, uint64_t u64Arg, PSUPDRVSESSION);
-VMMR0DECL(int)      VMMR0TermVM(PVM pVM, PGVM pGVM);
+#if defined(IN_RING0) || defined(DOXYGEN_RUNNING)
+VMMR0DECL(int)       VMMR0EntryInt(PVM pVM, VMMR0OPERATION enmOperation, void *pvArg);
+VMMR0DECL(void)      VMMR0EntryFast(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation);
+VMMR0DECL(int)       VMMR0EntryEx(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation, PSUPVMMR0REQHDR pReq, uint64_t u64Arg, PSUPDRVSESSION);
+VMMR0_INT_DECL(int)  VMMR0TermVM(PVM pVM, PGVM pGVM);
+VMMR0_INT_DECL(bool) VMMR0IsLongJumpArmed(PVMCPU pVCpu);
+VMMR0_INT_DECL(bool) VMMR0IsInRing3LongJump(PVMCPU pVCpu);
+VMMR0_INT_DECL(int)  VMMR0ThreadCtxHookCreateForEmt(PVMCPU pVCpu);
+VMMR0_INT_DECL(void) VMMR0ThreadCtxHookDestroyForEmt(PVMCPU pVCpu);
+VMMR0_INT_DECL(void) VMMR0ThreadCtxHookDisable(PVMCPU pVCpu);
+VMMR0_INT_DECL(bool) VMMR0ThreadCtxHookIsEnabled(PVMCPU pVCpu);
 
-#ifdef LOG_ENABLED
-VMMR0DECL(void)     VMMR0LogFlushDisable(PVMCPU pVCpu);
-VMMR0DECL(void)     VMMR0LogFlushEnable(PVMCPU pVCpu);
-#else
-#define             VMMR0LogFlushDisable(pVCpu)     do { } while(0)
-#define             VMMR0LogFlushEnable(pVCpu)      do { } while(0)
-#endif
+# ifdef LOG_ENABLED
+VMMR0_INT_DECL(void) VMMR0LogFlushDisable(PVMCPU pVCpu);
+VMMR0_INT_DECL(void) VMMR0LogFlushEnable(PVMCPU pVCpu);
+VMMR0_INT_DECL(bool) VMMR0IsLogFlushDisabled(PVMCPU pVCpu);
+# else
+#  define            VMMR0LogFlushDisable(pVCpu)     do { } while(0)
+#  define            VMMR0LogFlushEnable(pVCpu)      do { } while(0)
+#  define            VMMR0IsLogFlushDisabled(pVCpu)  (true)
+# endif /* LOG_ENABLED */
+#endif /* IN_RING0 */
 
 /** @} */
 
 
-#ifdef IN_RC
-/** @defgroup grp_vmm_rc    The VMM Raw-Mode Context API
- * @ingroup grp_vmm
+#if defined(IN_RC) || defined(DOXYGEN_RUNNING)
+/** @defgroup grp_vmm_api_rc    The VMM Raw-Mode Context API
  * @{
  */
-VMMRCDECL(int)      VMMGCEntry(PVM pVM, unsigned uOperation, unsigned uArg, ...);
-VMMRCDECL(void)     VMMGCGuestToHost(PVM pVM, int rc);
-VMMRCDECL(void)     VMMGCLogFlushIfFull(PVM pVM);
+VMMRCDECL(int)      VMMRCEntry(PVM pVM, unsigned uOperation, unsigned uArg, ...);
+VMMRCDECL(void)     VMMRCGuestToHost(PVM pVM, int rc);
+VMMRCDECL(void)     VMMRCLogFlushIfFull(PVM pVM);
 /** @} */
 #endif /* IN_RC */
 
-#if defined(IN_RC) || defined(IN_RING0)
-/** @defgroup grp_vmm_rz    The VMM Raw-Mode and Ring-0 Context API
- * @ingroup grp_vmm
+#if defined(IN_RC) || defined(IN_RING0) || defined(DOXYGEN_RUNNING)
+/** @defgroup grp_vmm_api_rz    The VMM Raw-Mode and Ring-0 Context API
  * @{
  */
 VMMRZDECL(int)      VMMRZCallRing3(PVM pVM, PVMCPU pVCpu, VMMCALLRING3 enmOperation, uint64_t uArg);
@@ -522,10 +559,15 @@ VMMRZDECL(int)      VMMRZCallRing3NoCpu(PVM pVM, VMMCALLRING3 enmOperation, uint
 VMMRZDECL(void)     VMMRZCallRing3Disable(PVMCPU pVCpu);
 VMMRZDECL(void)     VMMRZCallRing3Enable(PVMCPU pVCpu);
 VMMRZDECL(bool)     VMMRZCallRing3IsEnabled(PVMCPU pVCpu);
+VMMRZDECL(int)      VMMRZCallRing3SetNotification(PVMCPU pVCpu, R0PTRTYPE(PFNVMMR0CALLRING3NOTIFICATION) pfnCallback, RTR0PTR pvUser);
+VMMRZDECL(void)     VMMRZCallRing3RemoveNotification(PVMCPU pVCpu);
+VMMRZDECL(bool)     VMMRZCallRing3IsNotificationSet(PVMCPU pVCpu);
 /** @} */
 #endif
 
 
+/** @} */
+
 /** @} */
 RT_C_DECLS_END
 
