@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2011-2012 Oracle Corporation
+ * Copyright (C) 2011-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -287,7 +287,7 @@ static int VBoxDispInitDevice(PVBOXDISPDEV pDev, DEVMODEW *pdm, GDIINFO *pGdiInf
 
     pGdiInfo->ulNumPalReg = (pDev->mode.ulBitsPerPel==8) ? (1<<pDev->mode.ulBitsPerPel) : 0;
 
-    /* @todo: might want to implement IOCTL_VIDEO_QUERY_COLOR_CAPABILITIES in miniport driver
+    /** @todo might want to implement IOCTL_VIDEO_QUERY_COLOR_CAPABILITIES in miniport driver
      *        and query host for this info there
      */
     VBOXDISPSETCIEC(pGdiInfo->ciDevice.Red, 6700, 3300, 0);
@@ -392,7 +392,7 @@ static int VBoxDispInitDevice(PVBOXDISPDEV pDev, DEVMODEW *pdm, GDIINFO *pGdiInf
  */
 BOOL DrvEnableDriver(ULONG iEngineVersion, ULONG cj, PDRVENABLEDATA pded)
 {
-    /*@todo: can't link with hal.lib
+    /** @todo can't link with hal.lib
     int irc = RTR0Init(0);
     if (RT_FAILURE(irc))
     {
@@ -499,6 +499,7 @@ VBoxDispDrvEnablePDEV(DEVMODEW *pdm, LPWSTR pwszLogAddress, ULONG cPat, HSURF *p
                       ULONG cjDevInfo, DEVINFO  *pdi,
                       HDEV  hdev, PWSTR pwszDeviceName, HANDLE hDriver)
 {
+    RT_NOREF(hdev);
     PVBOXDISPDEV pDev = NULL;
     GDIINFO gdiInfo;
     DEVINFO devInfo;
@@ -605,7 +606,7 @@ HSURF APIENTRY VBoxDispDrvEnableSurface(DHPDEV dhpdev)
     VBOX_WARNRC_RETV(rc, NULL);
 
     /* Clear mapped memory, to avoid garbage while video mode is switching */
-    /* @todo: VIDEO_MODE_NO_ZERO_MEMORY does nothing in miniport's IOCTL_VIDEO_SET_CURRENT_MODE*/
+    /** @todo VIDEO_MODE_NO_ZERO_MEMORY does nothing in miniport's IOCTL_VIDEO_SET_CURRENT_MODE*/
     memset(pDev->memInfo.FrameBufferBase, 0, pDev->mode.ulHeight * abs(pDev->mode.lScanlineStride));
 
     /* Allocate memory for pointer attrs */
@@ -675,13 +676,15 @@ HSURF APIENTRY VBoxDispDrvEnableSurface(DHPDEV dhpdev)
             iFormat = BMF_32BPP;
             break;
         }
+        default:
+            AssertMsgFailedReturn(("ulBitsPerPel=%#x\n", pDev->mode.ulBitsPerPel), NULL);
     }
 
     size.cx = pDev->mode.ulWidth;
     size.cy = pDev->mode.ulHeight;
 
     pDev->surface.hBitmap = EngCreateBitmap(size, pDev->mode.lScanlineStride, iFormat,
-                                            pDev->mode.lScanlineStride>0 ? BMF_TOPDOWN:0,
+                                            pDev->mode.lScanlineStride > 0 ? BMF_TOPDOWN:0,
                                             pDev->memInfo.FrameBufferBase);
     if (!pDev->surface.hBitmap)
     {
@@ -782,6 +785,7 @@ VBoxDispDrvRealizeBrush(BRUSHOBJ *pbo, SURFOBJ *psoTarget, SURFOBJ *psoPattern,
 
 ULONG APIENTRY VBoxDispDrvDitherColor(DHPDEV dhpdev, ULONG iMode, ULONG rgb, ULONG *pul)
 {
+    RT_NOREF(dhpdev, iMode, rgb, pul);
     ULONG rc;
     LOGF_ENTER();
 
@@ -1043,6 +1047,7 @@ ULONG APIENTRY VBoxDispDrvEscape(SURFOBJ *pso, ULONG iEsc, ULONG cjIn, PVOID pvI
 /* Obsolete, NT4 specific. Called to set display offset in virtual desktop */
 BOOL APIENTRY VBoxDispDrvOffset(SURFOBJ* pso, LONG x, LONG y, FLONG flReserved)
 {
+    RT_NOREF(flReserved);
     PVBOXDISPDEV pDev = (PVBOXDISPDEV)pso->dhpdev;
     LOGF(("%x %x %x\n", x, y, flReserved));
 
