@@ -1,11 +1,10 @@
 /* $Id: VBoxDispInternal.h $ */
-
 /** @file
  * VBox XPDM Display driver, internal header
  */
 
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,7 +23,7 @@
 #include <VBox/log.h>
 #include <iprt/err.h>
 #include <iprt/assert.h>
-#include <windef.h>
+#include <iprt/win/windef.h>
 #include <wingdi.h>
 #include <winddi.h>
 #include <ntddvdeo.h>
