@@ -1,11 +1,10 @@
 /* $Id: VBoxMPVModes.cpp $ */
-
 /** @file
  * VBox WDDM Miniport driver
  */
 
 /*
- * Copyright (C) 2014 Oracle Corporation
+ * Copyright (C) 2014-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -310,7 +309,6 @@ int voxWddmVModesInitForTarget(PVBOXMP_DEVEXT pExt, VBOXWDDM_VMODES *pModes, uin
      * Give up on the first error encountered.
      */
     VBOXMPCMNREGISTRY Registry;
-    int fPrefSet=0;
     VP_STATUS vpRc;
 
     vpRc = VBoxMPCmnRegInit(pExt, &Registry);
@@ -613,7 +611,7 @@ NTSTATUS VBoxWddmChildStatusReportReconnected(PVBOXMP_DEVEXT pDevExt, uint32_t i
         if (RT_SUCCESS(rc))
         {
             Status = KeWaitForSingleObject(&Event, Executive, KernelMode, FALSE, NULL);
-            Assert(Status == STATUS_SUCCESS);
+            AssertNtStatusSuccess(Status);
             return STATUS_SUCCESS;
         }
 
@@ -623,7 +621,7 @@ NTSTATUS VBoxWddmChildStatusReportReconnected(PVBOXMP_DEVEXT pDevExt, uint32_t i
     }
     else
     {
-        /* @todo: try flushing.. */
+        /** @todo try flushing.. */
         WARN(("vboxVdmaCBufDrCreate returned NULL"));
         Status = STATUS_INSUFFICIENT_RESOURCES;
     }
