@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2011-2012 Oracle Corporation
+ * Copyright (C) 2011-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -40,6 +40,7 @@
  */
 BOOLEAN VBoxMPResetDevice(PVBOXMP_DEVEXT pExt, PSTATUS_BLOCK pStatus)
 {
+    RT_NOREF(pStatus);
     LOGF_ENTER();
 
     if (pExt->iDevice>0)
@@ -57,7 +58,7 @@ BOOLEAN VBoxMPResetDevice(PVBOXMP_DEVEXT pExt, PSTATUS_BLOCK pStatus)
 #endif
 
     /* Tell the host that we no longer support graphics in the additions */
-    /* @todo VBoxMPSetGraphicsCap(FALSE); */
+    /** @todo VBoxMPSetGraphicsCap(FALSE); */
 
     LOGF_LEAVE();
     return TRUE;
@@ -218,7 +219,7 @@ BOOLEAN VBoxMPSetCurrentMode(PVBOXMP_DEVEXT pExt, PVIDEO_MODE pMode, PSTATUS_BLO
     VBoxVideoSetModeRegisters((USHORT)pModeInfo->VisScreenWidth, (USHORT)pModeInfo->VisScreenHeight,
                               (USHORT)pModeInfo->VisScreenWidth, (USHORT)pModeInfo->BitsPerPlane, 0, 0, 0);
 
-    /*@todo read back from port to check if mode switch was successful */
+    /** @todo read back from port to check if mode switch was successful */
 
     LOGF_LEAVE();
     return TRUE;
@@ -277,6 +278,7 @@ BOOLEAN VBoxMPQueryAvailModes(PVBOXMP_DEVEXT pExt, PVIDEO_MODE_INFORMATION pMode
  */
 BOOLEAN VBoxMPSetColorRegisters(PVBOXMP_DEVEXT pExt, PVIDEO_CLUT pClut, PSTATUS_BLOCK pStatus)
 {
+    RT_NOREF(pExt);
     LONG entry;
 
     LOGF_ENTER();
@@ -410,6 +412,7 @@ BOOLEAN VBoxMPQueryPointerPosition(PVBOXMP_DEVEXT pExt, PVIDEO_POINTER_POSITION
  */
 BOOLEAN VBoxMPQueryPointerCapabilities(PVBOXMP_DEVEXT pExt, PVIDEO_POINTER_CAPABILITIES pCaps, PSTATUS_BLOCK pStatus)
 {
+    RT_NOREF(pExt);
     LOGF_ENTER();
 
     pStatus->Information = sizeof(VIDEO_POINTER_CAPABILITIES);
@@ -419,8 +422,8 @@ BOOLEAN VBoxMPQueryPointerCapabilities(PVBOXMP_DEVEXT pExt, PVIDEO_POINTER_CAPAB
     pCaps->MaxWidth  = 64;
     pCaps->MaxHeight = 64;
     /* Not used by our display driver */
-    pCaps->HWPtrBitmapStart = -1;
-    pCaps->HWPtrBitmapEnd   = -1;
+    pCaps->HWPtrBitmapStart = ~(ULONG)0;
+    pCaps->HWPtrBitmapEnd   = ~(ULONG)0;
 
     LOGF_LEAVE();
     return TRUE;
