@@ -166,7 +166,7 @@ HRESULT VBoxCredProvCredential::RTUTF16ToUnicodeA(PUNICODE_STRING pUnicodeDest,
     AssertPtrReturn(pUnicodeDest, E_POINTER);
     AssertPtrReturn(pwszSource,   E_POINTER);
 
-    size_t cbLen = (RTUtf16Len(pwszSource) + 1 /* Trailing zero */) * sizeof(WCHAR);
+    size_t cbLen = RTUtf16Len(pwszSource) * sizeof(RTUTF16);
 
     pUnicodeDest->Buffer = (LPWSTR)CoTaskMemAlloc(cbLen);
 
@@ -195,7 +195,7 @@ void VBoxCredProvCredential::UnicodeStringFree(PUNICODE_STRING pUnicode)
         Assert(pUnicode->MaximumLength);
 
         /* Make sure to wipe contents before free'ing. */
-        RTMemWipeThoroughly(pUnicode->Buffer, pUnicode->MaximumLength * sizeof(WCHAR), 3 /* Passes */);
+        RTMemWipeThoroughly(pUnicode->Buffer, pUnicode->MaximumLength /* MaximumLength is bytes! */, 3 /* Passes */);
 
         CoTaskMemFree(pUnicode->Buffer);
         pUnicode->Buffer = NULL;
@@ -317,12 +317,12 @@ HRESULT VBoxCredProvCredential::kerberosLogonSerialize(const KERB_INTERACTIVE_LO
      * credentials.
      */
     DWORD cbLogon = sizeof(KERB_INTERACTIVE_UNLOCK_LOGON)
-                  + pLogonIn->LogonDomainName.Length +
-                  + pLogonIn->UserName.Length +
+                  + pLogonIn->LogonDomainName.Length
+                  + pLogonIn->UserName.Length
                   + pLogonIn->Password.Length;
 
-#ifdef DEBUG
-    VBoxCredProvVerbose(3, "VBoxCredProvCredential::AllocateLogonPackage: Allocating %ld bytes (%d bytes credentials)\n",
+#ifdef DEBUG /* Do not reveal any hints to credential data in release mode. */
+    VBoxCredProvVerbose(1, "VBoxCredProvCredential::AllocateLogonPackage: Allocating %ld bytes (%zu bytes credentials)\n",
                         cbLogon, cbLogon - sizeof(KERB_INTERACTIVE_UNLOCK_LOGON));
 #endif
 
@@ -330,6 +330,9 @@ HRESULT VBoxCredProvCredential::kerberosLogonSerialize(const KERB_INTERACTIVE_LO
     if (!pLogon)
         return E_OUTOFMEMORY;
 
+    /* Make sure to zero everything first. */
+    RT_BZERO(pLogon, cbLogon);
+
     /* Let our byte buffer point to the end of our allocated structure so that it can
      * be used to store the credential data sequentially in a binary blob
      * (without terminating \0). */
@@ -338,17 +341,19 @@ HRESULT VBoxCredProvCredential::kerberosLogonSerialize(const KERB_INTERACTIVE_LO
     /* The buffer of the packed destination string does not contain the actual
      * string content but a relative offset starting at the given
      * KERB_INTERACTIVE_UNLOCK_LOGON structure. */
-#define KERB_CRED_INIT_PACKED(StringDst, StringSrc, LogonOffset)         \
-    StringDst.Length         = StringSrc.Length;                         \
-    StringDst.MaximumLength  = StringSrc.Length;                         \
-    StringDst.Buffer         = (PWSTR)pbBuffer;                          \
-    memcpy(StringDst.Buffer, StringSrc.Buffer, StringDst.Length);        \
-    StringDst.Buffer         = (PWSTR)(pbBuffer - (PBYTE)LogonOffset);   \
-    pbBuffer                += StringDst.Length;
-
-    RT_BZERO(&pLogon->LogonId, sizeof(LUID));
+#define KERB_CRED_INIT_PACKED(StringDst, StringSrc, LogonOffset)             \
+    StringDst.Length         = StringSrc.Length;                             \
+    StringDst.MaximumLength  = StringSrc.Length;                             \
+    if (StringDst.Length)                                                    \
+    {                                                                        \
+        StringDst.Buffer         = (PWSTR)pbBuffer;                          \
+        memcpy(StringDst.Buffer, StringSrc.Buffer, StringDst.Length);        \
+        StringDst.Buffer         = (PWSTR)(pbBuffer - (PBYTE)LogonOffset);   \
+        pbBuffer                += StringDst.Length;                         \
+    }
 
     KERB_INTERACTIVE_LOGON *pLogonOut = &pLogon->Logon;
+
     pLogonOut->MessageType = pLogonIn->MessageType;
 
     KERB_CRED_INIT_PACKED(pLogonOut->LogonDomainName, pLogonIn->LogonDomainName, pLogon);
@@ -1037,6 +1042,10 @@ HRESULT VBoxCredProvCredential::GetSerialization(CREDENTIAL_PROVIDER_GET_SERIALI
 
                             /* We're done -- let the logon UI know. */
                             *pcpGetSerializationResponse = CPGSR_RETURN_CREDENTIAL_FINISHED;
+
+                            VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: Finished for user '%ls' (domain '%s')\n",
+                                                m_apwszCredentials[VBOXCREDPROV_FIELDID_USERNAME],
+                                                m_apwszCredentials[VBOXCREDPROV_FIELDID_DOMAINNAME]);
                         }
                         else
                             VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: LsaLookupAuthenticationPackage failed with ntStatus=%ld\n", s);
@@ -1057,7 +1066,7 @@ HRESULT VBoxCredProvCredential::GetSerialization(CREDENTIAL_PROVIDER_GET_SERIALI
     else
         VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: kerberosLogonCreate failed with hr=0x%08x\n", hr);
 
-    VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization returned hr=0x%08x\n", hr);
+    VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization returned hr=0x%08x\n", hr);
     return hr;
 }
 
