@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,7 +15,7 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#include <windows.h>
+#include <iprt/win/windows.h>
 
 #include <iprt/semaphore.h>
 #include <iprt/string.h>
@@ -144,6 +144,7 @@ RTTHREAD gThreadPoller = NIL_RTTHREAD;
  */
 static DECLCALLBACK(int) credentialsPoller(RTTHREAD ThreadSelf, void *pvUser)
 {
+    RT_NOREF(pvUser);
     VBoxGINAVerbose(0, "VBoxGINA::credentialsPoller\n");
 
     do
