@@ -1,10 +1,10 @@
 /* $Id: DBGPlugInDiggers.cpp $ */
 /** @file
- * DBGPlugInDiggers - Debugger and Guest OS Digger Plug-in.
+ * DbfPlugInDiggers - Debugger and Guest OS Digger Plug-in.
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +16,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_DBGC
 #include <VBox/dbg.h>
 #include <VBox/vmm/dbgf.h>
@@ -27,10 +27,11 @@
 #include <VBox/err.h>
 
 
-DECLEXPORT(int) DBGCPlugInEntry(DBGCPLUGINOP enmOperation, PVM pVM, uintptr_t uArg)
+DECLEXPORT(int) DbgPlugInEntry(DBGFPLUGINOP enmOperation, PUVM pUVM, uintptr_t uArg)
 {
     static PCDBGFOSREG s_aPlugIns[] =
     {
+        &g_DBGDiggerDarwin,
         //&g_DBGDiggerFreeBSD,
         &g_DBGDiggerLinux,
         &g_DBGDiggerOS2,
@@ -40,30 +41,30 @@ DECLEXPORT(int) DBGCPlugInEntry(DBGCPLUGINOP enmOperation, PVM pVM, uintptr_t uA
 
     switch (enmOperation)
     {
-        case DBGCPLUGINOP_INIT:
+        case DBGFPLUGINOP_INIT:
         {
             if (uArg != VBOX_VERSION)
                 return VERR_VERSION_MISMATCH;
 
             for (unsigned i = 0; i < RT_ELEMENTS(s_aPlugIns); i++)
             {
-                int rc = DBGFR3OSRegister(pVM, s_aPlugIns[i]);
+                int rc = DBGFR3OSRegister(pUVM, s_aPlugIns[i]);
                 if (RT_FAILURE(rc))
                 {
                     AssertRC(rc);
                     while (i-- > 0)
-                        DBGFR3OSDeregister(pVM, s_aPlugIns[i]);
+                        DBGFR3OSDeregister(pUVM, s_aPlugIns[i]);
                     return rc;
                 }
             }
             return VINF_SUCCESS;
         }
 
-        case DBGCPLUGINOP_TERM:
+        case DBGFPLUGINOP_TERM:
         {
             for (unsigned i = 0; i < RT_ELEMENTS(s_aPlugIns); i++)
             {
-                int rc = DBGFR3OSDeregister(pVM, s_aPlugIns[i]);
+                int rc = DBGFR3OSDeregister(pUVM, s_aPlugIns[i]);
                 AssertRC(rc);
             }
             return VINF_SUCCESS;
