@@ -170,8 +170,7 @@ static int drvHostALSAAudioClose(snd_pcm_t **pphPCM)
     int rc2 = snd_pcm_close(*pphPCM);
     if (rc2)
     {
-        LogRel(("ALSA: Closing PCM descriptor failed: %s\n",
-                 snd_strerror(rc2)));
+        LogRel(("ALSA: Closing PCM descriptor failed: %s\n", snd_strerror(rc2)));
         rc = VERR_GENERAL_FAILURE; /** @todo */
     }
     else
@@ -385,8 +384,7 @@ static int drvHostALSAAudioOpen(bool fIn,
         const char *pszDev = fIn ? s_ALSAConf.pcm_name_in : s_ALSAConf.pcm_name_out;
         if (!pszDev)
         {
-            LogRel(("ALSA: Invalid or no %s device name set\n",
-                    fIn ? "input" : "output"));
+            LogRel(("ALSA: Invalid or no %s device name set\n", fIn ? "input" : "output"));
             rc = VERR_INVALID_PARAMETER;
             break;
         }
@@ -396,19 +394,19 @@ static int drvHostALSAAudioOpen(bool fIn,
                                SND_PCM_NONBLOCK);
         if (err < 0)
         {
-            LogRel(("ALSA: Failed to open \"%s\" as %s: %s\n", pszDev,
-                    fIn ? "ADC" : "DAC", snd_strerror(err)));
+            LogRel(("ALSA: Failed to open \"%s\" as %s device: %s\n", pszDev, fIn ? "input" : "output", snd_strerror(err)));
             rc = VERR_AUDIO_BACKEND_INIT_FAILED;
             break;
         }
 
+        LogRel(("ALSA: Using %s device \"%s\"\n", fIn ? "input" : "output", pszDev));
+
         snd_pcm_hw_params_t *pHWParms;
         snd_pcm_hw_params_alloca(&pHWParms); /** @todo Check for successful allocation? */
         err = snd_pcm_hw_params_any(phPCM, pHWParms);
         if (err < 0)
         {
-            LogRel(("ALSA: Failed to initialize hardware parameters: %s\n",
-                    snd_strerror(err)));
+            LogRel(("ALSA: Failed to initialize hardware parameters: %s\n", snd_strerror(err)));
             rc = VERR_AUDIO_BACKEND_INIT_FAILED;
             break;
         }
@@ -425,8 +423,7 @@ static int drvHostALSAAudioOpen(bool fIn,
         err = snd_pcm_hw_params_set_format(phPCM, pHWParms, pCfgReq->fmt);
         if (err < 0)
         {
-            LogRel(("ALSA: Failed to set audio format to %d: %s\n",
-                    pCfgReq->fmt, snd_strerror(err)));
+            LogRel(("ALSA: Failed to set audio format to %d: %s\n", pCfgReq->fmt, snd_strerror(err)));
             rc = VERR_AUDIO_BACKEND_INIT_FAILED;
             break;
         }
@@ -434,8 +431,7 @@ static int drvHostALSAAudioOpen(bool fIn,
         err = snd_pcm_hw_params_set_rate_near(phPCM, pHWParms, &uFreq, 0);
         if (err < 0)
         {
-            LogRel(("ALSA: Failed to set frequency to %dHz: %s\n",
-                    pCfgReq->freq, snd_strerror(err)));
+            LogRel(("ALSA: Failed to set frequency to %uHz: %s\n", pCfgReq->freq, snd_strerror(err)));
             rc = VERR_AUDIO_BACKEND_INIT_FAILED;
             break;
         }
@@ -665,6 +661,7 @@ static int drvHostALSAAudioOpen(bool fIn,
 static void drvHostALSAAudioErrorHandler(const char *file, int line, const char *function,
                                          int err, const char *fmt, ...)
 {
+    RT_NOREF(file, line, function, err, fmt);
     /** @todo Implement me! */
 }
 #endif
@@ -705,8 +702,7 @@ static int drvHostALSAAudioRecover(snd_pcm_t *phPCM)
     int err = snd_pcm_prepare(phPCM);
     if (err < 0)
     {
-        LogFunc(("Failed to recover stream %p: %s\n",
-                 phPCM, snd_strerror(err)));
+        LogFunc(("Failed to recover stream %p: %s\n", phPCM, snd_strerror(err)));
         return VERR_ACCESS_DENIED; /** @todo Find a better rc. */
     }
 
@@ -720,8 +716,7 @@ static int drvHostALSAAudioResume(snd_pcm_t *phPCM)
     int err = snd_pcm_resume(phPCM);
     if (err < 0)
     {
-        LogFunc(("Failed to resume stream %p: %s\n",
-                 phPCM, snd_strerror(err)));
+        LogFunc(("Failed to resume stream %p: %s\n", phPCM, snd_strerror(err)));
         return VERR_ACCESS_DENIED; /** @todo Find a better rc. */
     }
 
@@ -736,18 +731,16 @@ static int drvHostALSAAudioStreamCtl(snd_pcm_t *phPCM, bool fPause)
         err = snd_pcm_drop(phPCM);
         if (err < 0)
         {
-            LogFlow(("Error stopping stream %p: %s\n",
-                     phPCM, snd_strerror(err)));
+            LogRel(("ALSA: Error stopping stream %p: %s\n", phPCM, snd_strerror(err)));
             return VERR_ACCESS_DENIED;
         }
     }
     else
     {
-        err = snd_pcm_prepare (phPCM);
+        err = snd_pcm_prepare(phPCM);
         if (err < 0)
         {
-            LogFlow(("Error preparing stream %p: %s\n",
-                     phPCM, snd_strerror(err)));
+            LogRel(("ALSA: Error preparing stream %p: %s\n", phPCM, snd_strerror(err)));
             return VERR_ACCESS_DENIED;
         }
     }
@@ -855,6 +848,7 @@ static DECLCALLBACK(int) drvHostALSAAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PP
                 }
 
                 case -EAGAIN:
+                {
                     /*
                      * Don't set error here because EAGAIN means there are no further frames
                      * available at the moment, try later. As we might have read some frames
@@ -862,6 +856,7 @@ static DECLCALLBACK(int) drvHostALSAAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PP
                      */
                     cbToRead = 0;
                     break;
+                }
 
                 case -EPIPE:
                 {
@@ -874,10 +869,11 @@ static DECLCALLBACK(int) drvHostALSAAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PP
                 }
 
                 default:
+                {
                     LogFunc(("Failed to read input frames: %s\n", snd_strerror(cRead)));
-                    rc = VERR_AUDIO_BACKEND_INIT_FAILED;
-                    VERR_GENERAL_FAILURE; /** @todo Fudge! */
+                    rc = VERR_GENERAL_FAILURE; /** @todo Fudge! */
                     break;
+                }
             }
         }
         else
@@ -1080,12 +1076,13 @@ static DECLCALLBACK(int) drvHostALSAAudioFiniOut(PPDMIHOSTAUDIO pInterface, PPDM
 }
 
 static DECLCALLBACK(int) drvHostALSAAudioInitOut(PPDMIHOSTAUDIO pInterface,
-                                                 PPDMAUDIOHSTSTRMOUT pHstStrmOut, PPDMAUDIOSTREAMCFG pCfg,
+                                                 PPDMAUDIOHSTSTRMOUT pHstStrmOut,
+                                                 PPDMAUDIOSTREAMCFG pCfgReq, PPDMAUDIOSTREAMCFG pCfgAcq,
                                                  uint32_t *pcSamples)
 {
-    NOREF(pInterface);
+    RT_NOREF(pInterface, pCfgAcq);
     AssertPtrReturn(pHstStrmOut, VERR_INVALID_POINTER);
-    AssertPtrReturn(pCfg, VERR_INVALID_POINTER);
+    AssertPtrReturn(pCfgReq,     VERR_INVALID_POINTER);
 
     PALSAAUDIOSTREAMOUT pThisStrmOut = (PALSAAUDIOSTREAMOUT)pHstStrmOut;
     snd_pcm_t *phPCM = NULL;
@@ -1095,9 +1092,9 @@ static DECLCALLBACK(int) drvHostALSAAudioInitOut(PPDMIHOSTAUDIO pInterface,
     do
     {
         ALSAAUDIOSTREAMCFG req;
-        req.fmt         = drvHostALSAAudioFmtToALSA(pCfg->enmFormat);
-        req.freq        = pCfg->uHz;
-        req.nchannels   = pCfg->cChannels;
+        req.fmt         = drvHostALSAAudioFmtToALSA(pCfgReq->enmFormat);
+        req.freq        = pCfgReq->uHz;
+        req.nchannels   = pCfgReq->cChannels;
         req.period_size = s_ALSAConf.period_size_out;
         req.buffer_size = s_ALSAConf.buffer_size_out;
 
@@ -1150,13 +1147,13 @@ static DECLCALLBACK(int) drvHostALSAAudioInitOut(PPDMIHOSTAUDIO pInterface,
 }
 
 static DECLCALLBACK(int) drvHostALSAAudioInitIn(PPDMIHOSTAUDIO pInterface,
-                                                PPDMAUDIOHSTSTRMIN pHstStrmIn, PPDMAUDIOSTREAMCFG pCfg,
-                                                PDMAUDIORECSOURCE enmRecSource,
-                                                uint32_t *pcSamples)
+                                                PPDMAUDIOHSTSTRMIN pHstStrmIn,
+                                                PPDMAUDIOSTREAMCFG pCfgReq, PPDMAUDIOSTREAMCFG pCfgAcq,
+                                                PDMAUDIORECSOURCE enmRecSource, uint32_t *pcSamples)
 {
-    NOREF(pInterface);
+    RT_NOREF(pInterface, pCfgAcq, enmRecSource);
     AssertPtrReturn(pHstStrmIn, VERR_INVALID_POINTER);
-    AssertPtrReturn(pCfg, VERR_INVALID_POINTER);
+    AssertPtrReturn(pCfgReq,    VERR_INVALID_POINTER);
 
     int rc;
 
@@ -1166,9 +1163,9 @@ static DECLCALLBACK(int) drvHostALSAAudioInitIn(PPDMIHOSTAUDIO pInterface,
     do
     {
         ALSAAUDIOSTREAMCFG req;
-        req.fmt         = drvHostALSAAudioFmtToALSA(pCfg->enmFormat);
-        req.freq        = pCfg->uHz;
-        req.nchannels   = pCfg->cChannels;
+        req.fmt         = drvHostALSAAudioFmtToALSA(pCfgReq->enmFormat);
+        req.freq        = pCfgReq->uHz;
+        req.nchannels   = pCfgReq->cChannels;
         req.period_size = s_ALSAConf.period_size_in;
         req.buffer_size = s_ALSAConf.buffer_size_in;
 
@@ -1294,10 +1291,13 @@ static DECLCALLBACK(int) drvHostALSAAudioGetConf(PPDMIHOSTAUDIO pInterface, PPDM
     NOREF(pInterface);
     AssertPtrReturn(pCfg, VERR_INVALID_POINTER);
 
-    pCfg->cbStreamOut = sizeof(ALSAAUDIOSTREAMOUT);
-    pCfg->cbStreamIn = sizeof(ALSAAUDIOSTREAMIN);
-    pCfg->cMaxHstStrmsOut = INT_MAX;
-    pCfg->cMaxHstStrmsIn = INT_MAX;
+    pCfg->cbStreamIn      = sizeof(ALSAAUDIOSTREAMIN);
+    pCfg->cbStreamOut     = sizeof(ALSAAUDIOSTREAMOUT);
+
+    /* ALSA only allows one input and one output used at a time for
+     * the selected device. */
+    pCfg->cMaxHstStrmsIn  = 1;
+    pCfg->cMaxHstStrmsOut = 1;
 
     return VINF_SUCCESS;
 }
@@ -1327,6 +1327,7 @@ static DECLCALLBACK(void *) drvHostALSAAudioQueryInterface(PPDMIBASE pInterface,
  */
 static DECLCALLBACK(int) drvHostAlsaAudioConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
 {
+    RT_NOREF(pCfg, fFlags);
     PDRVHOSTALSAAUDIO pThis = PDMINS_2_DATA(pDrvIns, PDRVHOSTALSAAUDIO);
     LogRel(("Audio: Initializing ALSA driver\n"));
 
@@ -1393,6 +1394,7 @@ const PDMDRVREG g_DrvHostALSAAudio =
     PDM_DRVREG_VERSION
 };
 
+#if 0 // unused
 static struct audio_option alsa_options[] =
 {
     {"DACSizeInUsec", AUD_OPT_BOOL, &s_ALSAConf.size_in_usec_out,
@@ -1420,4 +1422,4 @@ static struct audio_option alsa_options[] =
 
     NULL
 };
-
+#endif
