@@ -1,7 +1,7 @@
 /* $Id: dnsproxy.c $ */
 
 /*
- * Copyright (C) 2009-2013 Oracle Corporation
+ * Copyright (C) 2009-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -10,15 +10,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * The contents of this file may alternatively be used under the terms
- * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
- * CDDL are applicable instead of those of the GPL.
- *
- * You may elect to license modified versions of this file under the
- * terms and conditions of either the GPL or the CDDL or both.
  */
 
 /*
@@ -128,10 +119,17 @@ timeout(PNATState pData, struct socket *so, void *arg)
     struct dns_entry *de;
     /* be paranoid */
     AssertPtrReturnVoid(arg);
-    de = TAILQ_PREV(req->dns_server, dns_list_head, de_list);
-    /* here we should check if we reached the end of the DNS server list */
-    if (de == NULL)
+
+    if (   req->dnsgen != pData->dnsgen
+        || req->dns_server == NULL
+        || (de = TAILQ_PREV(req->dns_server, dns_list_head, de_list)) == NULL)
     {
+        if (req->dnsgen != pData->dnsgen)
+        {
+            /* XXX: Log2 */
+            LogRel(("NAT: dnsproxy: timeout: req %p dnsgen %u != %u on %R[natsock]\n",
+                    req, req->dnsgen, pData->dnsgen, so));
+        }
         hash_remove_request(pData, req);
         RTMemFree(req);
         ++removed_queries;
@@ -139,34 +137,16 @@ timeout(PNATState pData, struct socket *so, void *arg)
     }
     else
     {
-        /* Note: so will be deleted after we create new one (so1)
-         * to make attempt on the next server.
-         */
         struct ip *ip;
         struct udphdr *udp;
         int iphlen;
-        struct socket *so1 = socreate();
         struct mbuf *m = NULL;
         char *data;
 
-        if (so1 == NULL)
-        {
-            LogRel(("NAT: can't create DNS socket\n"));
-            goto socket_clean_up;
-        }
-
-        if(udp_attach(pData, so1) == -1)
-        {
-            LogRel(("NAT: can't attach udp socket\n"));
-            sofree(pData, so1);
-            goto socket_clean_up;
-        }
-
         m = slirpDnsMbufAlloc(pData);
         if (m == NULL)
         {
             LogRel(("NAT: Can't allocate mbuf\n"));
-            udp_detach(pData, so1);
             goto socket_clean_up;
         }
 
@@ -186,23 +166,22 @@ timeout(PNATState pData, struct socket *so, void *arg)
         ip->ip_dst.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_DNS);
         udp->uh_dport = ntohs(53);
         udp->uh_sport = so->so_lport;
+
         memcpy(data, req->byte, req->nbyte); /* coping initial req */
 
-        so1->so_laddr = so->so_laddr;
-        so1->so_lport = so->so_lport;
-        so1->so_faddr = so->so_faddr;
-        so1->so_fport = so->so_fport;
-
+        /* req points to so->so_timeout_arg */
         req->dns_server = de;
-        so1->so_timeout_arg = req;
-        so1->so_timeout = timeout;
 
-        dnsproxy_query(pData, so1, m, iphlen);
+        /* expiration will be bumped in dnsproxy_query */
+
+        dnsproxy_query(pData, so, m, iphlen);
+        /* should we free so->so_m ? */
+        return;
     }
- 
- socket_clean_up:   
+
+ socket_clean_up:
     /* This socket (so) will be detached, so we need to remove timeout(&_arg) references
-     * before leave 
+     * before leave
      */
     so->so_timeout = NULL;
     so->so_timeout_arg = NULL;
@@ -262,19 +241,6 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
         ++dropped_queries;
         return;
     }
-#else
-    ip = mtod(m, struct ip *);
-    udp = (struct udphdr *)(m->m_data + iphlen);
-
-    fromaddr.sin_addr.s_addr = ip->ip_src.s_addr;
-    fromaddr.sin_port = udp->uh_sport;
-    fromaddr.sin_family = AF_INET;
-
-    iphlen += sizeof (struct udphdr);
-    byte = m->m_len - iphlen;  /* size of IP header + udp header size */
-    /* the validness of ip and udp header has been already checked so we shouldn't care if */
-    buf = m->m_data + iphlen;
-#endif
 
     /* check for minimum dns packet length */
     if (byte < 12) {
@@ -283,7 +249,6 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
         return;
     }
 
-#ifndef VBOX
     /* allocate new request */
     if ((req = calloc(1, sizeof(struct request))) == NULL) {
         LogRel(("calloc failed\n"));
@@ -294,50 +259,7 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
     req->id = QUERYID;
     memcpy(&req->client, &fromaddr, sizeof(struct sockaddr_in));
     memcpy(&req->clientid, &buf[0], 2);
-#else
-    /* allocate new request */
-    req = so->so_timeout_arg; /* in slirp we might re-send the query*/
-    if (req == NULL)
-    {
-        if ((req = RTMemAllocZ(sizeof(struct request) + byte)) == NULL) {
-            LogRel(("calloc failed\n"));
-            ++dropped_queries;
-            return;
-        }
-    }
 
-    /* fill the request structure */
-    if (so->so_timeout_arg == NULL)
-    {
-        req->id = QUERYID;
-        memcpy(&req->client, &fromaddr, sizeof(struct sockaddr_in));
-        memcpy(&req->clientid, &buf[0], 2);
-        req->dns_server = TAILQ_LAST(&pData->pDnsList, dns_list_head);
-        if (req->dns_server == NULL)
-        {
-            static int fail_counter = 0;
-            RTMemFree(req);
-            /** @todo: This is completely bogus, fail_counter will always be 0. */
-            if (fail_counter == 0)
-                LogRel(("NAT/dnsproxy: Empty DNS entry (suppressed 100 times)\n"));
-            else
-                fail_counter = (fail_counter == 100 ? 0 : fail_counter + 1);
-            return;
-
-        }
-        retransmit = 0;
-        so->so_timeout = timeout;
-        so->so_timeout_arg = req;
-        req->nbyte = byte;
-        memcpy(req->byte, buf, byte); /* copying original request */
-    }
-    else
-    {
-        retransmit = 1;
-    }
-#endif
-
-#ifndef VBOX
     /* where is this query coming from? */
     if (is_internal(pData, fromaddr.sin_addr)) {
         req->recursion = RD(buf);
@@ -350,17 +272,10 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
 
     /* insert it into the hash table */
     hash_add_request(pData, req);
-#else
-    req->recursion = 0;
-    DPRINTF(("External query RD=%d\n", RD(buf)));
-    if (retransmit == 0)
-        hash_add_request(pData, req);
-#endif
 
     /* overwrite the original query id */
     memcpy(&buf[0], &req->id, 2);
 
-#ifndef VBOX
     if (req->recursion) {
 
         /* recursive queries timeout in 90s */
@@ -394,30 +309,149 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
             ++dropped_queries;
             return;
         }
+        ++authoritative_queries;
+    }
 
-#else
-        so->so_expire = curtime + recursive_timeout * 1000; /* let's slirp to care about expiration */
-        memset(&addr, 0, sizeof(struct sockaddr_in));
-        addr.sin_family = AF_INET;
-        addr.sin_addr.s_addr  = req->dns_server->de_addr.s_addr;
-        addr.sin_port = htons(53);
-        so->so_expire = curtime + recursive_timeout * 1000; /* let's slirp to care about expiration */
-        /* send it to our authoritative server */
-        Log2(("NAT: request will be sent to %RTnaipv4 on %R[natsock]\n", addr.sin_addr, so));
-        if ((byte = sendto(so->s, buf, (unsigned int)byte, 0,
-                    (struct sockaddr *)&addr,
-                    sizeof(struct sockaddr_in))) == -1) {
-            LogRel(("sendto failed: %s\n", strerror(errno)));
+#else /* VBOX */
+    AssertPtr(pData);
+
+    /* m->m_data points to IP header */
+#if 0
+    /* XXX: for some reason it make gdb ill,
+     * it good to have this assert here with assumption above.
+     */
+    M_ASSERTPKTHDR(m);
+#endif
+
+    ip = mtod(m, struct ip *);
+    udp = (struct udphdr *)(m->m_data + iphlen);
+
+    fromaddr.sin_addr.s_addr = ip->ip_src.s_addr;
+    fromaddr.sin_port = udp->uh_sport;
+    fromaddr.sin_family = AF_INET;
+
+    /* iphlen equals to lenght of ip header */
+    Assert(iphlen == sizeof(struct ip));
+    iphlen += sizeof (struct udphdr);
+
+    byte = m->m_len - iphlen;
+    buf = m->m_data + iphlen;
+
+    /* check for minimum dns packet length */
+    if (byte < 12) {
+        LogRel(("NAT: Query too short from %RTnaipv4\n", fromaddr.sin_addr));
+        ++dropped_queries;
+        return;
+    }
+
+    req = so->so_timeout_arg;
+
+    if (!req)
+    {
+
+        Assert(!so->so_timeout_arg);
+
+        if ((req = RTMemAllocZ(sizeof(struct request) + byte)) == NULL)
+        {
+            LogRel(("NAT: calloc failed\n"));
             ++dropped_queries;
             return;
         }
-        so->so_state = SS_ISFCONNECTED; /* now it's selected */
-        Log2(("NAT: request was sent to %RTnaipv4 on %R[natsock]\n", addr.sin_addr, so));
-#endif
-        ++authoritative_queries;
-#ifndef VBOX
+
+        req->id = QUERYID;
+        memcpy(&req->client, &fromaddr, sizeof(struct sockaddr_in));
+        memcpy(&req->clientid, &buf[0], 2);
+        req->dns_server = TAILQ_LAST(&pData->pDnsList, dns_list_head);
+        req->dnsgen = pData->dnsgen;
+        if (req->dns_server == NULL)
+        {
+            RTMemFree(req);
+            return;
+        }
+        retransmit = 0;
+        so->so_timeout = timeout;
+        so->so_timeout_arg = req;
+        req->nbyte = byte;
+        memcpy(req->byte, buf, byte); /* copying original request */
     }
-#endif
+    else
+    {
+        if (req->dnsgen != pData->dnsgen)
+        {
+            /* XXX: Log2 */
+            LogRel(("NAT: dnsproxy: query: req %p dnsgen %u != %u on %R[natsock]\n",
+                    req, req->dnsgen, pData->dnsgen, so));
+            /*
+             * XXX: TODO: this probably requires more cleanup.
+             * Cf. XXX comment for sendto() failure below, but that
+             * error leg is probably untested since ~never taken.
+             */
+            ++dropped_queries;
+            return;
+        }
+        retransmit = 1;
+    }
+
+    req->recursion = 0;
+
+    DPRINTF(("External query RD=%d\n", RD(buf)));
+
+    if (retransmit == 0)
+        hash_add_request(pData, req);
+
+
+    /* overwrite the original query id */
+    memcpy(&buf[0], &req->id, 2);
+
+    /* let's slirp to care about expiration */
+    so->so_expire = curtime + recursive_timeout * 1000;
+
+    memset(&addr, 0, sizeof(struct sockaddr_in));
+    addr.sin_family = AF_INET;
+    if (req->dns_server->de_addr.s_addr == (pData->special_addr.s_addr | RT_H2N_U32_C(CTL_ALIAS))) {
+        /* undo loopback remapping done in get_dns_addr_domain() */
+        addr.sin_addr.s_addr = RT_N2H_U32_C(INADDR_LOOPBACK);
+    }
+    else {
+        addr.sin_addr.s_addr = req->dns_server->de_addr.s_addr;
+    }
+    addr.sin_port = htons(53);
+
+    /* send it to our authoritative server */
+    Log2(("NAT: request will be %ssent to %RTnaipv4 on %R[natsock]\n",
+          retransmit ? "re" : "", addr.sin_addr, so));
+
+    byte = sendto(so->s, buf, (unsigned int)byte, 0,
+                  (struct sockaddr *)&addr,
+                  sizeof(struct sockaddr_in));
+    if (byte == -1)
+    {
+        /* XXX: is it really enough? */
+        LogRel(("NAT: sendto failed: %s\n", strerror(errno)));
+        ++dropped_queries;
+        return;
+    }
+
+    so->so_state = SS_ISFCONNECTED; /* now it's selected */
+    Log2(("NAT: request was %ssent to %RTnaipv4 on %R[natsock]\n",
+          retransmit ? "re" : "", addr.sin_addr, so));
+
+    ++authoritative_queries;
+
+# if 0
+    /* XXX: this stuff for _debugging_ only,
+     * first enforce guest to send next request
+     * and second for faster getting timeout callback
+     * other option is adding couple entries in resolv.conf with
+     * invalid nameservers.
+     *
+     * For testing purposes could be used
+     * namebench -S -q 10000 -m random or -m chunk
+     */
+    /* RTThreadSleep(3000); */
+    /* curtime += 300; */
+# endif
+#endif /* VBOX */
 }
 
 /* do_answer -- Process a packet coming from our authoritative or recursive
@@ -451,13 +485,6 @@ dnsproxy_answer(PNATState pData, struct socket *so, struct mbuf *m)
         ++dropped_answers;
         return;
     }
-#else
-    char *buf;
-    int byte;
-    struct request *query = NULL;
-    byte = m->m_len;
-    buf = mtod(m, char *);
-#endif
 
     /* check for minimum dns packet length */
     if (byte < 12) {
@@ -467,51 +494,104 @@ dnsproxy_answer(PNATState pData, struct socket *so, struct mbuf *m)
     }
 
     /* find corresponding query */
-#ifdef VBOX
-    if ((query = hash_find_request(pData, *((unsigned short *)buf))) == NULL) {
-        ++late_answers;
-        /* Probably, this request wasn't serviced by
-         * dnsproxy so we won't care about it here*/
-        so->so_expire = curtime + SO_EXPIREFAST;
-        Log2(("NAT: query wasn't found\n"));
-        return;
-    }
-    so->so_timeout = NULL;
-    so->so_timeout_arg = NULL;
-#else
     if ((query = hash_find_request(pData, *((unsigned short *)&buf))) == NULL) {
         ++late_answers;
         return;
     }
     event_del(&query->timeout);
-#endif
+
     hash_remove_request(pData, query);
 
     /* restore original query id */
     memcpy(&buf[0], &query->clientid, 2);
 
-#ifndef VBOX
-    /* Slirp: will send mbuf to guest by itself */
-    /* send answer back to querying host */
     if (sendto(sock_query, buf, (unsigned int)byte, 0,
                 (struct sockaddr *)&query->client,
                 sizeof(struct sockaddr_in)) == -1) {
         LogRel(("sendto failed: %s\n", strerror(errno)));
         ++dropped_answers;
-    } else
+    }
+    else
         ++answered_queries;
 
     free(query);
-#else
+#else /* VBOX */
+
+    char *buf = NULL;
+    int byte = 0;
+    struct request *query = NULL;
+
+    AssertPtr(pData);
+
+    /* XXX: mbuf->data points to ??? */
+    byte = m->m_len;
+    buf = mtod(m, char *);
+
+    /* check for minimum dns packet length */
+    if (byte < 12) {
+        LogRel(("NAT: Answer too short\n"));
+        ++dropped_answers;
+        return;
+    }
+
+    /* find corresponding query (XXX: but see below) */
+    query = hash_find_request(pData, *((unsigned short *)buf));
+
+    if (query == NULL)
+    {
+        /* XXX: if we haven't found anything for this request ...
+         * What we are expecting later?
+         */
+        ++late_answers;
+        so->so_expire = curtime + SO_EXPIREFAST;
+        Log2(("NAT: query wasn't found\n"));
+        return;
+    }
+
+    /*
+     * XXX: The whole hash thing is pretty meaningless right now since
+     * we use a separate socket for each request, so we already know
+     * the answer.
+     *
+     * If the answer is not what we expect it to be, then it's
+     * probably a stray or malicious reply and we'd better not free a
+     * query owned by some other socket - that would cause
+     * use-after-free later on.
+     */
+    if (query != so->so_timeout_arg)
+        return;
+
+    so->so_timeout = NULL;
+    so->so_timeout_arg = NULL;
+
+    hash_remove_request(pData, query);
+
+    /* restore original query id */
+    memcpy(&buf[0], &query->clientid, 2);
+
     ++answered_queries;
 
     RTMemFree(query);
-#endif
+#endif /* VBOX */
 }
 
+
+#ifdef VBOX
+int
+dnsproxy_init(PNATState pData)
+{
+    /* globals initialization */
+    authoritative_port = 53;
+    authoritative_timeout = 10;
+    recursive_port = 53;
+    recursive_timeout = 2;
+    stats_timeout = 3600;
+    dns_port = 53;
+    return 0;
+}
+#else /* !VBOX */
 /* main -- dnsproxy main function
  */
-#ifndef VBOX
 int
 main(int argc, char *argv[])
 {
@@ -671,17 +751,4 @@ main(int argc, char *argv[])
     return 0;
 
 }
-#else
-int
-dnsproxy_init(PNATState pData)
-{
-    /* globals initialization */
-    authoritative_port = 53;
-    authoritative_timeout = 10;
-    recursive_port = 53;
-    recursive_timeout = 2;
-    stats_timeout = 3600;
-    dns_port = 53;
-    return 0;
-}
 #endif
