@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,17 +16,15 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <VBox/com/com.h>
 #include <VBox/com/string.h>
 #include <VBox/com/Guid.h>
 #include <VBox/com/array.h>
 #include <VBox/com/ErrorInfo.h>
 #include <VBox/com/errorprint.h>
-#include <VBox/com/EventQueue.h>
-
 #include <VBox/com/VirtualBox.h>
 
 #include <iprt/ctype.h>
@@ -66,7 +64,7 @@ static unsigned parseNum(const char *psz, unsigned cMaxNum, const char *name)
 
 unsigned int getMaxNics(IVirtualBox* vbox, IMachine* mach)
 {
-    ComPtr <ISystemProperties> info;
+    ComPtr<ISystemProperties> info;
     ChipsetType_T aChipset;
     ULONG NetworkAdapterCount = 0;
     HRESULT rc;
@@ -83,33 +81,38 @@ unsigned int getMaxNics(IVirtualBox* vbox, IMachine* mach)
 }
 
 
-int handleControlVM(HandlerArg *a)
+RTEXITCODE handleControlVM(HandlerArg *a)
 {
     using namespace com;
+    bool fNeedsSaving = false;
     HRESULT rc;
 
     if (a->argc < 2)
         return errorSyntax(USAGE_CONTROLVM, "Not enough parameters");
 
     /* try to find the given machine */
-    ComPtr <IMachine> machine;
+    ComPtr<IMachine> machine;
     CHECK_ERROR(a->virtualBox, FindMachine(Bstr(a->argv[0]).raw(),
                                            machine.asOutParam()));
     if (FAILED(rc))
-        return 1;
+        return RTEXITCODE_FAILURE;
 
     /* open a session for the VM */
-    CHECK_ERROR_RET(machine, LockMachine(a->session, LockType_Shared), 1);
+    CHECK_ERROR_RET(machine, LockMachine(a->session, LockType_Shared), RTEXITCODE_FAILURE);
+
+    ComPtr<IConsole> console;
+    ComPtr<IMachine> sessionMachine;
 
     do
     {
         /* get the associated console */
-        ComPtr<IConsole> console;
         CHECK_ERROR_BREAK(a->session, COMGETTER(Console)(console.asOutParam()));
         /* ... and session machine */
-        ComPtr<IMachine> sessionMachine;
         CHECK_ERROR_BREAK(a->session, COMGETTER(Machine)(sessionMachine.asOutParam()));
 
+        if (!console)
+            return RTMsgErrorExit(RTEXITCODE_FAILURE, "Machine '%s' is not currently running", a->argv[0]);
+
         /* which command? */
         if (!strcmp(a->argv[1], "pause"))
         {
@@ -171,7 +174,7 @@ int handleControlVM(HandlerArg *a)
                 break;
             }
 
-            ClipboardMode_T mode;
+            ClipboardMode_T mode = ClipboardMode_Disabled; /* Shut up MSC */
             if (!strcmp(a->argv[2], "disabled"))
                 mode = ClipboardMode_Disabled;
             else if (!strcmp(a->argv[2], "hosttoguest"))
@@ -188,26 +191,28 @@ int handleControlVM(HandlerArg *a)
             if (SUCCEEDED(rc))
             {
                 CHECK_ERROR_BREAK(sessionMachine, COMSETTER(ClipboardMode)(mode));
+                if (SUCCEEDED(rc))
+                    fNeedsSaving = true;
             }
         }
         else if (!strcmp(a->argv[1], "draganddrop"))
         {
             if (a->argc <= 1 + 1)
             {
-                errorArgument("Missing argument to '%s'. Expected drag'n'drop mode.", a->argv[1]);
+                errorArgument("Missing argument to '%s'. Expected drag and drop mode.", a->argv[1]);
                 rc = E_FAIL;
                 break;
             }
 
-            DragAndDropMode_T mode;
+            DnDMode_T mode = DnDMode_Disabled; /* Shup up MSC. */
             if (!strcmp(a->argv[2], "disabled"))
-                mode = DragAndDropMode_Disabled;
+                mode = DnDMode_Disabled;
             else if (!strcmp(a->argv[2], "hosttoguest"))
-                mode = DragAndDropMode_HostToGuest;
+                mode = DnDMode_HostToGuest;
             else if (!strcmp(a->argv[2], "guesttohost"))
-                mode = DragAndDropMode_GuestToHost;
+                mode = DnDMode_GuestToHost;
             else if (!strcmp(a->argv[2], "bidirectional"))
-                mode = DragAndDropMode_Bidirectional;
+                mode = DnDMode_Bidirectional;
             else
             {
                 errorArgument("Invalid '%s' argument '%s'.", a->argv[1], a->argv[2]);
@@ -215,7 +220,9 @@ int handleControlVM(HandlerArg *a)
             }
             if (SUCCEEDED(rc))
             {
-                CHECK_ERROR_BREAK(sessionMachine, COMSETTER(DragAndDropMode)(mode));
+                CHECK_ERROR_BREAK(sessionMachine, COMSETTER(DnDMode)(mode));
+                if (SUCCEEDED(rc))
+                    fNeedsSaving = true;
             }
         }
         else if (!strcmp(a->argv[1], "poweroff"))
@@ -257,7 +264,7 @@ int handleControlVM(HandlerArg *a)
             }
 
             ComPtr<IProgress> progress;
-            CHECK_ERROR(console, SaveState(progress.asOutParam()));
+            CHECK_ERROR(sessionMachine, SaveState(progress.asOutParam()));
             if (FAILED(rc))
             {
                 if (!fPaused)
@@ -347,7 +354,6 @@ int handleControlVM(HandlerArg *a)
         {
             /* Get the number of network adapters */
             ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, sessionMachine);
-
             unsigned n = parseNum(&a->argv[1][12], NetworkAdapterCount, "NIC");
             if (!n)
             {
@@ -379,6 +385,8 @@ int handleControlVM(HandlerArg *a)
                     rc = E_FAIL;
                     break;
                 }
+                if (SUCCEEDED(rc))
+                    fNeedsSaving = true;
             }
         }
         /* here the order in which strncmp is called is important
@@ -414,7 +422,7 @@ int handleControlVM(HandlerArg *a)
                 {
                     if (a->argv[2])
                     {
-                        CHECK_ERROR_RET(adapter, COMSETTER(TraceFile)(Bstr(a->argv[2]).raw()), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(TraceFile)(Bstr(a->argv[2]).raw()), RTEXITCODE_FAILURE);
                     }
                     else
                     {
@@ -422,6 +430,8 @@ int handleControlVM(HandlerArg *a)
                         rc = E_FAIL;
                         break;
                     }
+                    if (SUCCEEDED(rc))
+                        fNeedsSaving = true;
                 }
                 else
                     RTMsgError("The NIC %d is currently disabled and thus its tracefile can't be changed", n);
@@ -431,7 +441,6 @@ int handleControlVM(HandlerArg *a)
         {
             /* Get the number of network adapters */
             ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, sessionMachine);
-
             unsigned n = parseNum(&a->argv[1][8], NetworkAdapterCount, "NIC");
             if (!n)
             {
@@ -456,11 +465,11 @@ int handleControlVM(HandlerArg *a)
                 {
                     if (!strcmp(a->argv[2], "on"))
                     {
-                        CHECK_ERROR_RET(adapter, COMSETTER(TraceEnabled)(TRUE), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(TraceEnabled)(TRUE), RTEXITCODE_FAILURE);
                     }
                     else if (!strcmp(a->argv[2], "off"))
                     {
-                        CHECK_ERROR_RET(adapter, COMSETTER(TraceEnabled)(FALSE), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(TraceEnabled)(FALSE), RTEXITCODE_FAILURE);
                     }
                     else
                     {
@@ -468,6 +477,8 @@ int handleControlVM(HandlerArg *a)
                         rc = E_FAIL;
                         break;
                     }
+                    if (SUCCEEDED(rc))
+                        fNeedsSaving = true;
                 }
                 else
                     RTMsgError("The NIC %d is currently disabled and thus its trace flag can't be changed", n);
@@ -478,7 +489,6 @@ int handleControlVM(HandlerArg *a)
         {
             /* Get the number of network adapters */
             ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, sessionMachine);
-            ComPtr<INATEngine> engine;
             unsigned n = parseNum(&a->argv[1][5], NetworkAdapterCount, "NIC");
             if (!n)
             {
@@ -500,6 +510,7 @@ int handleControlVM(HandlerArg *a)
                 rc = E_FAIL;
                 break;
             }
+            ComPtr<INATEngine> engine;
             CHECK_ERROR(adapter, COMGETTER(NATEngine)(engine.asOutParam()));
             if (!engine)
             {
@@ -564,14 +575,13 @@ int handleControlVM(HandlerArg *a)
                         RTStrToUInt16(strHostPort), Bstr(strGuestIp).raw(), RTStrToUInt16(strGuestPort)));
 #undef ITERATE_TO_NEXT_TERM
             }
-            /* commit changes */
             if (SUCCEEDED(rc))
-                CHECK_ERROR(sessionMachine, SaveSettings());
+                fNeedsSaving = true;
         }
         else if (!strncmp(a->argv[1], "nicproperty", 11))
         {
             /* Get the number of network adapters */
-            ULONG NetworkAdapterCount = getMaxNics(a->virtualBox,sessionMachine) ;
+            ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, sessionMachine);
             unsigned n = parseNum(&a->argv[1][11], NetworkAdapterCount, "NIC");
             if (!n)
             {
@@ -606,6 +616,8 @@ int handleControlVM(HandlerArg *a)
                             Bstr bstrName = pszProperty;
                             Bstr bstrValue = &pDelimiter[1];
                             CHECK_ERROR(adapter, SetProperty(bstrName.raw(), bstrValue.raw()));
+                            if (SUCCEEDED(rc))
+                                fNeedsSaving = true;
                         }
                         else
                         {
@@ -629,7 +641,7 @@ int handleControlVM(HandlerArg *a)
         else if (!strncmp(a->argv[1], "nicpromisc", 10))
         {
             /* Get the number of network adapters */
-            ULONG NetworkAdapterCount = getMaxNics(a->virtualBox,sessionMachine) ;
+            ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, sessionMachine);
             unsigned n = parseNum(&a->argv[1][10], NetworkAdapterCount, "NIC");
             if (!n)
             {
@@ -668,6 +680,8 @@ int handleControlVM(HandlerArg *a)
                     }
 
                     CHECK_ERROR(adapter, COMSETTER(PromiscModePolicy)(enmPromiscModePolicy));
+                    if (SUCCEEDED(rc))
+                        fNeedsSaving = true;
                 }
                 else
                     RTMsgError("The NIC %d is currently disabled and thus its promiscuous mode can't be changed", n);
@@ -676,7 +690,7 @@ int handleControlVM(HandlerArg *a)
         else if (!strncmp(a->argv[1], "nic", 3))
         {
             /* Get the number of network adapters */
-            ULONG NetworkAdapterCount = getMaxNics(a->virtualBox,sessionMachine) ;
+            ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, sessionMachine);
             unsigned n = parseNum(&a->argv[1][3], NetworkAdapterCount, "NIC");
             if (!n)
             {
@@ -701,15 +715,13 @@ int handleControlVM(HandlerArg *a)
                 {
                     if (!strcmp(a->argv[2], "null"))
                     {
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Null), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Null), RTEXITCODE_FAILURE);
                     }
                     else if (!strcmp(a->argv[2], "nat"))
                     {
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
                         if (a->argc == 4)
-                            CHECK_ERROR_RET(adapter, COMSETTER(NATNetwork)(Bstr(a->argv[3]).raw()), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_NAT), 1);
+                            CHECK_ERROR_RET(adapter, COMSETTER(NATNetwork)(Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_NAT), RTEXITCODE_FAILURE);
                     }
                     else if (  !strcmp(a->argv[2], "bridged")
                             || !strcmp(a->argv[2], "hostif")) /* backward compatibility */
@@ -720,9 +732,8 @@ int handleControlVM(HandlerArg *a)
                             rc = E_FAIL;
                             break;
                         }
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(BridgedInterface)(Bstr(a->argv[3]).raw()), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Bridged), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(BridgedInterface)(Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Bridged), RTEXITCODE_FAILURE);
                     }
                     else if (!strcmp(a->argv[2], "intnet"))
                     {
@@ -732,9 +743,8 @@ int handleControlVM(HandlerArg *a)
                             rc = E_FAIL;
                             break;
                         }
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(InternalNetwork)(Bstr(a->argv[3]).raw()), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Internal), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(InternalNetwork)(Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Internal), RTEXITCODE_FAILURE);
                     }
 #if defined(VBOX_WITH_NETFLT)
                     else if (!strcmp(a->argv[2], "hostonly"))
@@ -745,9 +755,8 @@ int handleControlVM(HandlerArg *a)
                             rc = E_FAIL;
                             break;
                         }
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(HostOnlyInterface)(Bstr(a->argv[3]).raw()), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_HostOnly), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(HostOnlyInterface)(Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_HostOnly), RTEXITCODE_FAILURE);
                     }
 #endif
                     else if (!strcmp(a->argv[2], "generic"))
@@ -758,9 +767,19 @@ int handleControlVM(HandlerArg *a)
                             rc = E_FAIL;
                             break;
                         }
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(GenericDriver)(Bstr(a->argv[3]).raw()), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Generic), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(GenericDriver)(Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Generic), RTEXITCODE_FAILURE);
+                    }
+                    else if (!strcmp(a->argv[2], "natnetwork"))
+                    {
+                        if (a->argc <= 3)
+                        {
+                            errorArgument("Missing argument to '%s'", a->argv[2]);
+                            rc = E_FAIL;
+                            break;
+                        }
+                        CHECK_ERROR_RET(adapter, COMSETTER(NATNetwork)(Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_NATNetwork), RTEXITCODE_FAILURE);
                     }
                     /** @todo obsolete, remove eventually */
                     else if (!strcmp(a->argv[2], "vde"))
@@ -771,9 +790,8 @@ int handleControlVM(HandlerArg *a)
                             rc = E_FAIL;
                             break;
                         }
-                        CHECK_ERROR_RET(adapter, COMSETTER(Enabled)(TRUE), 1);
-                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Generic), 1);
-                        CHECK_ERROR_RET(adapter, SetProperty(Bstr("name").raw(), Bstr(a->argv[3]).raw()), 1);
+                        CHECK_ERROR_RET(adapter, COMSETTER(AttachmentType)(NetworkAttachmentType_Generic), RTEXITCODE_FAILURE);
+                        CHECK_ERROR_RET(adapter, SetProperty(Bstr("name").raw(), Bstr(a->argv[3]).raw()), RTEXITCODE_FAILURE);
                     }
                     else
                     {
@@ -781,6 +799,8 @@ int handleControlVM(HandlerArg *a)
                         rc = E_FAIL;
                         break;
                     }
+                    if (SUCCEEDED(rc))
+                        fNeedsSaving = true;
                 }
                 else
                     RTMsgError("The NIC %d is currently disabled and thus its attachment type can't be changed", n);
@@ -817,6 +837,8 @@ int handleControlVM(HandlerArg *a)
                     rc = E_FAIL;
                     break;
                 }
+                if (SUCCEEDED(rc))
+                    fNeedsSaving = true;
             }
         }
         else if (   !strcmp(a->argv[1], "vrdeport")
@@ -845,6 +867,8 @@ int handleControlVM(HandlerArg *a)
                     ports = a->argv[2];
 
                 CHECK_ERROR_BREAK(vrdeServer, SetVRDEProperty(Bstr("TCP/Ports").raw(), ports.raw()));
+                if (SUCCEEDED(rc))
+                    fNeedsSaving = true;
             }
         }
         else if (   !strcmp(a->argv[1], "vrdevideochannelquality")
@@ -867,6 +891,8 @@ int handleControlVM(HandlerArg *a)
                 Bstr value = a->argv[2];
 
                 CHECK_ERROR(vrdeServer, SetVRDEProperty(Bstr("VideoChannel/Quality").raw(), value.raw()));
+                if (SUCCEEDED(rc))
+                    fNeedsSaving = true;
             }
         }
         else if (!strcmp(a->argv[1], "vrdeproperty"))
@@ -894,6 +920,8 @@ int handleControlVM(HandlerArg *a)
                         Bstr bstrName = pszProperty;
                         Bstr bstrValue = &pDelimiter[1];
                         CHECK_ERROR(vrdeServer, SetVRDEProperty(bstrName.raw(), bstrValue.raw()));
+                        if (SUCCEEDED(rc))
+                            fNeedsSaving = true;
                     }
                     else
                     {
@@ -922,20 +950,41 @@ int handleControlVM(HandlerArg *a)
                 rc = E_FAIL;
                 break;
             }
+            else if (a->argc == 4 || a->argc > 5)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Wrong number of arguments");
+                rc = E_FAIL;
+                break;
+            }
 
             bool attach = !strcmp(a->argv[1], "usbattach");
 
             Bstr usbId = a->argv[2];
-            if (Guid(usbId).isEmpty())
+            Bstr captureFilename;
+
+            if (a->argc == 5)
+            {
+                if (!strcmp(a->argv[3], "--capturefile"))
+                    captureFilename = a->argv[4];
+                else
+                {
+                    errorArgument("Invalid parameter '%s'", a->argv[3]);
+                    rc = E_FAIL;
+                    break;
+                }
+            }
+
+            Guid guid(usbId);
+            if (!guid.isValid())
             {
                 // assume address
                 if (attach)
                 {
-                    ComPtr <IHost> host;
+                    ComPtr<IHost> host;
                     CHECK_ERROR_BREAK(a->virtualBox, COMGETTER(Host)(host.asOutParam()));
                     SafeIfaceArray <IHostUSBDevice> coll;
                     CHECK_ERROR_BREAK(host, COMGETTER(USBDevices)(ComSafeArrayAsOutParam(coll)));
-                    ComPtr <IHostUSBDevice> dev;
+                    ComPtr<IHostUSBDevice> dev;
                     CHECK_ERROR_BREAK(host, FindUSBDeviceByAddress(Bstr(a->argv[2]).raw(),
                                                                    dev.asOutParam()));
                     CHECK_ERROR_BREAK(dev, COMGETTER(Id)(usbId.asOutParam()));
@@ -944,18 +993,24 @@ int handleControlVM(HandlerArg *a)
                 {
                     SafeIfaceArray <IUSBDevice> coll;
                     CHECK_ERROR_BREAK(console, COMGETTER(USBDevices)(ComSafeArrayAsOutParam(coll)));
-                    ComPtr <IUSBDevice> dev;
+                    ComPtr<IUSBDevice> dev;
                     CHECK_ERROR_BREAK(console, FindUSBDeviceByAddress(Bstr(a->argv[2]).raw(),
                                                                       dev.asOutParam()));
                     CHECK_ERROR_BREAK(dev, COMGETTER(Id)(usbId.asOutParam()));
                 }
             }
+            else if (guid.isZero())
+            {
+                errorArgument("Zero UUID argument '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
 
             if (attach)
-                CHECK_ERROR_BREAK(console, AttachUSBDevice(usbId.raw()));
+                CHECK_ERROR_BREAK(console, AttachUSBDevice(usbId.raw(), captureFilename.raw()));
             else
             {
-                ComPtr <IUSBDevice> dev;
+                ComPtr<IUSBDevice> dev;
                 CHECK_ERROR_BREAK(console, DetachUSBDevice(usbId.raw(),
                                                            dev.asOutParam()));
             }
@@ -1065,7 +1120,7 @@ int handleControlVM(HandlerArg *a)
                                                      Bstr(domain).raw(),
                                                      fAllowLocalLogon));
         }
-#if 0 /* TODO: review & remove */
+#if 0 /** @todo review & remove */
         else if (!strcmp(a->argv[1], "dvdattach"))
         {
             Bstr uuid;
@@ -1205,7 +1260,7 @@ int handleControlVM(HandlerArg *a)
                 break;
             }
             /* guest is running; update IGuest */
-            ComPtr <IGuest> pGuest;
+            ComPtr<IGuest> pGuest;
             rc = console->COMGETTER(Guest)(pGuest.asOutParam());
             if (SUCCEEDED(rc))
             {
@@ -1292,10 +1347,10 @@ int handleControlVM(HandlerArg *a)
                 break;
             }
             int vrc;
-            uint32_t displayIdx = 0;
+            uint32_t iScreen = 0;
             if (a->argc == 4)
             {
-                vrc = RTStrToUInt32Ex(a->argv[3], NULL, 0, &displayIdx);
+                vrc = RTStrToUInt32Ex(a->argv[3], NULL, 0, &iScreen);
                 if (vrc != VINF_SUCCESS)
                 {
                     errorArgument("Error parsing display number '%s'", a->argv[3]);
@@ -1312,25 +1367,372 @@ int handleControlVM(HandlerArg *a)
                 break;
             }
             ULONG width, height, bpp;
-            CHECK_ERROR_BREAK(pDisplay, GetScreenResolution(displayIdx, &width, &height, &bpp));
+            LONG xOrigin, yOrigin;
+            GuestMonitorStatus_T monitorStatus;
+            CHECK_ERROR_BREAK(pDisplay, GetScreenResolution(iScreen, &width, &height, &bpp, &xOrigin, &yOrigin, &monitorStatus));
             com::SafeArray<BYTE> saScreenshot;
-            CHECK_ERROR_BREAK(pDisplay, TakeScreenShotPNGToArray(displayIdx, width, height, ComSafeArrayAsOutParam(saScreenshot)));
+            CHECK_ERROR_BREAK(pDisplay, TakeScreenShotToArray(iScreen, width, height, BitmapFormat_PNG, ComSafeArrayAsOutParam(saScreenshot)));
             RTFILE pngFile = NIL_RTFILE;
             vrc = RTFileOpen(&pngFile, a->argv[2], RTFILE_O_OPEN_CREATE | RTFILE_O_WRITE | RTFILE_O_TRUNCATE | RTFILE_O_DENY_ALL);
             if (RT_FAILURE(vrc))
             {
-                RTMsgError("Failed to create file '%s'. rc=%Rrc", a->argv[2], vrc);
+                RTMsgError("Failed to create file '%s' (%Rrc)", a->argv[2], vrc);
                 rc = E_FAIL;
                 break;
             }
             vrc = RTFileWrite(pngFile, saScreenshot.raw(), saScreenshot.size(), NULL);
             if (RT_FAILURE(vrc))
             {
-                RTMsgError("Failed to write screenshot to file '%s'. rc=%Rrc", a->argv[2], vrc);
+                RTMsgError("Failed to write screenshot to file '%s' (%Rrc)", a->argv[2], vrc);
                 rc = E_FAIL;
             }
             RTFileClose(pngFile);
         }
+#ifdef VBOX_WITH_VPX
+        /*
+         * Note: Commands starting with "vcp" are the deprecated versions and are
+         *       kept to ensure backwards compatibility.
+         */
+        else if (   !strcmp(a->argv[1], "videocap")
+                 || !strcmp(a->argv[1], "vcpenabled"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+            if (!strcmp(a->argv[2], "on"))
+            {
+                CHECK_ERROR_RET(sessionMachine, COMSETTER(VideoCaptureEnabled)(TRUE), RTEXITCODE_FAILURE);
+            }
+            else if (!strcmp(a->argv[2], "off"))
+            {
+                CHECK_ERROR_RET(sessionMachine, COMSETTER(VideoCaptureEnabled)(FALSE), RTEXITCODE_FAILURE);
+            }
+            else
+            {
+                errorArgument("Invalid state '%s'", Utf8Str(a->argv[2]).c_str());
+                rc = E_FAIL;
+                break;
+            }
+        }
+        else if (   !strcmp(a->argv[1], "videocapscreens")
+                 || !strcmp(a->argv[1], "vcpscreens"))
+        {
+            ULONG cMonitors = 64;
+            CHECK_ERROR_BREAK(machine, COMGETTER(MonitorCount)(&cMonitors));
+            com::SafeArray<BOOL> saScreens(cMonitors);
+            if (   a->argc == 3
+                && !strcmp(a->argv[2], "all"))
+            {
+                /* enable all screens */
+                for (unsigned i = 0; i < cMonitors; i++)
+                    saScreens[i] = true;
+            }
+            else if (   a->argc == 3
+                     && !strcmp(a->argv[2], "none"))
+            {
+                /* disable all screens */
+                for (unsigned i = 0; i < cMonitors; i++)
+                    saScreens[i] = false;
+
+                /** @todo r=andy What if this is specified? */
+            }
+            else
+            {
+                /* enable selected screens */
+                for (unsigned i = 0; i < cMonitors; i++)
+                    saScreens[i] = false;
+                for (int i = 2; SUCCEEDED(rc) && i < a->argc; i++)
+                {
+                    uint32_t iScreen;
+                    int vrc = RTStrToUInt32Ex(a->argv[i], NULL, 0, &iScreen);
+                    if (vrc != VINF_SUCCESS)
+                    {
+                        errorArgument("Error parsing display number '%s'", a->argv[i]);
+                        rc = E_FAIL;
+                        break;
+                    }
+                    if (iScreen >= cMonitors)
+                    {
+                        errorArgument("Invalid screen ID specified '%u'", iScreen);
+                        rc = E_FAIL;
+                        break;
+                    }
+                    saScreens[iScreen] = true;
+                }
+            }
+
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureScreens)(ComSafeArrayAsInParam(saScreens)));
+        }
+        else if (   !strcmp(a->argv[1], "videocapfile")
+                 || !strcmp(a->argv[1], "vcpfile"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureFile)(Bstr(a->argv[3]).raw()));
+        }
+        else if (!strcmp(a->argv[1], "videocapres"))
+        {
+            if (a->argc != 4)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing width '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureWidth)(uVal));
+
+            vrc = RTStrToUInt32Ex(a->argv[3], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing height '%s'", a->argv[3]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureHeight)(uVal));
+        }
+        else if (!strcmp(a->argv[1], "vcpwidth")) /* Deprecated; keeping for compatibility. */
+        {
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing width '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureWidth)(uVal));
+        }
+        else if (!strcmp(a->argv[1], "vcpheight")) /* Deprecated; keeping for compatibility. */
+        {
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing height '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureHeight)(uVal));
+        }
+        else if (   !strcmp(a->argv[1], "videocaprate")
+                 || !strcmp(a->argv[1], "vcprate"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing rate '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureRate)(uVal));
+        }
+        else if (   !strcmp(a->argv[1], "videocapfps")
+                 || !strcmp(a->argv[1], "vcpfps"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing FPS '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureFPS)(uVal));
+        }
+        else if (   !strcmp(a->argv[1], "videocapmaxtime")
+                 || !strcmp(a->argv[1], "vcpmaxtime"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing maximum time '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureMaxTime)(uVal));
+        }
+        else if (   !strcmp(a->argv[1], "videocapmaxsize")
+                 || !strcmp(a->argv[1], "vcpmaxsize"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            uint32_t uVal;
+            int vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
+            if (RT_FAILURE(vrc))
+            {
+                errorArgument("Error parsing maximum file size '%s'", a->argv[2]);
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureMaxFileSize)(uVal));
+        }
+        else if (   !strcmp(a->argv[1], "videocapopts")
+                 || !strcmp(a->argv[1], "vcpoptions"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                rc = E_FAIL;
+                break;
+            }
+
+            CHECK_ERROR_BREAK(sessionMachine, COMSETTER(VideoCaptureOptions)(Bstr(a->argv[3]).raw()));
+        }
+#endif /* VBOX_WITH_VPX */
+        else if (!strcmp(a->argv[1], "webcam"))
+        {
+            if (a->argc < 3)
+            {
+                errorArgument("Missing argument to '%s'", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+
+            ComPtr<IEmulatedUSB> pEmulatedUSB;
+            CHECK_ERROR_BREAK(console, COMGETTER(EmulatedUSB)(pEmulatedUSB.asOutParam()));
+            if (!pEmulatedUSB)
+            {
+                RTMsgError("Guest not running");
+                rc = E_FAIL;
+                break;
+            }
+
+            if (!strcmp(a->argv[2], "attach"))
+            {
+                Bstr path("");
+                if (a->argc >= 4)
+                    path = a->argv[3];
+                Bstr settings("");
+                if (a->argc >= 5)
+                    settings = a->argv[4];
+                CHECK_ERROR_BREAK(pEmulatedUSB, WebcamAttach(path.raw(), settings.raw()));
+            }
+            else if (!strcmp(a->argv[2], "detach"))
+            {
+                Bstr path("");
+                if (a->argc >= 4)
+                    path = a->argv[3];
+                CHECK_ERROR_BREAK(pEmulatedUSB, WebcamDetach(path.raw()));
+            }
+            else if (!strcmp(a->argv[2], "list"))
+            {
+                com::SafeArray <BSTR> webcams;
+                CHECK_ERROR_BREAK(pEmulatedUSB, COMGETTER(Webcams)(ComSafeArrayAsOutParam(webcams)));
+                for (size_t i = 0; i < webcams.size(); ++i)
+                {
+                    RTPrintf("%ls\n", webcams[i][0]? webcams[i]: Bstr("default").raw());
+                }
+            }
+            else
+            {
+                errorArgument("Invalid argument to '%s'", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+        }
+        else if (!strcmp(a->argv[1], "addencpassword"))
+        {
+            if (   a->argc != 4
+                && a->argc != 6)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                break;
+            }
+
+            BOOL fRemoveOnSuspend = FALSE;
+            if (a->argc == 6)
+            {
+                if (   strcmp(a->argv[4], "--removeonsuspend")
+                    || (   strcmp(a->argv[5], "yes")
+                        && strcmp(a->argv[5], "no")))
+                {
+                    errorSyntax(USAGE_CONTROLVM, "Invalid parameters");
+                    break;
+                }
+                if (!strcmp(a->argv[5], "yes"))
+                    fRemoveOnSuspend = TRUE;
+            }
+
+            Bstr bstrPwId(a->argv[2]);
+            Utf8Str strPassword;
+
+            if (!RTStrCmp(a->argv[3], "-"))
+            {
+                /* Get password from console. */
+                RTEXITCODE rcExit = readPasswordFromConsole(&strPassword, "Enter password:");
+                if (rcExit == RTEXITCODE_FAILURE)
+                    break;
+            }
+            else
+            {
+                RTEXITCODE rcExit = readPasswordFile(a->argv[3], &strPassword);
+                if (rcExit == RTEXITCODE_FAILURE)
+                {
+                    RTMsgError("Failed to read new password from file");
+                    break;
+                }
+            }
+
+            CHECK_ERROR_BREAK(console, AddDiskEncryptionPassword(bstrPwId.raw(), Bstr(strPassword).raw(), fRemoveOnSuspend));
+        }
+        else if (!strcmp(a->argv[1], "removeencpassword"))
+        {
+            if (a->argc != 3)
+            {
+                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
+                break;
+            }
+            Bstr bstrPwId(a->argv[2]);
+            CHECK_ERROR_BREAK(console, RemoveDiskEncryptionPassword(bstrPwId.raw()));
+        }
+        else if (!strcmp(a->argv[1], "removeallencpasswords"))
+        {
+            CHECK_ERROR_BREAK(console, ClearAllDiskEncryptionPasswords());
+        }
         else
         {
             errorSyntax(USAGE_CONTROLVM, "Invalid parameter '%s'", a->argv[1]);
@@ -1338,7 +1740,11 @@ int handleControlVM(HandlerArg *a)
         }
     } while (0);
 
+    /* The client has to trigger saving the state explicitely. */
+    if (fNeedsSaving)
+        CHECK_ERROR(sessionMachine, SaveSettings());
+
     a->session->UnlockMachine();
 
-    return SUCCEEDED(rc) ? 0 : 1;
+    return SUCCEEDED(rc) ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
 }
