@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -20,10 +20,12 @@
 
 /* Qt includes: */
 #include <QMetaType>
+#include <QObject>
 
 /* Other VBox includes: */
 #include <iprt/cdefs.h>
 
+
 /** Extra-data namespace. */
 namespace UIExtraDataDefs
 {
@@ -31,6 +33,9 @@ namespace UIExtraDataDefs
       * @{ */
         /** VBox language ID. */
         extern const char* GUI_LanguageId;
+
+        /** Holds event handling type. */
+        extern const char* GUI_EventHandlingType;
     /** @} */
 
     /** @name Messaging
@@ -134,12 +139,12 @@ namespace UIExtraDataDefs
         extern const char* GUI_PreventSnapshotOperations;
         /** Holds whether this machine is first time started. */
         extern const char* GUI_FirstRun;
-#ifndef Q_WS_MAC
+#ifndef VBOX_WS_MAC
         /** Except Mac OS X: Holds redefined machine-window icon names. */
         extern const char* GUI_MachineWindowIcons;
         /** Except Mac OS X: Holds redefined machine-window name postfix. */
         extern const char* GUI_MachineWindowNamePostfix;
-#endif /* !Q_WS_MAC */
+#endif /* !VBOX_WS_MAC */
         /** Prefix used by composite extra-data keys,
           * which holds normal machine-window geometry per screen-index. */
         extern const char* GUI_LastNormalWindowPosition;
@@ -148,10 +153,10 @@ namespace UIExtraDataDefs
         extern const char* GUI_LastScaleWindowPosition;
         /** Holds machine-window geometry maximized state flag. */
         extern const char* GUI_Geometry_State_Max;
-#ifndef Q_WS_MAC
+#ifndef VBOX_WS_MAC
         /** Holds Runtime UI menu-bar availability status. */
         extern const char* GUI_MenuBar_Enabled;
-#endif /* !Q_WS_MAC */
+#endif /* !VBOX_WS_MAC */
         /** Holds restricted Runtime UI menu types. */
         extern const char* GUI_RestrictedRuntimeMenus;
         /** Holds restricted Runtime UI action types for 'Application' menu. */
@@ -168,10 +173,10 @@ namespace UIExtraDataDefs
         /** Holds restricted Runtime UI action types for Debugger menu. */
         extern const char* GUI_RestrictedRuntimeDebuggerMenuActions;
 #endif /* VBOX_WITH_DEBUGGER_GUI */
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
         /** Mac OS X: Holds restricted Runtime UI action types for 'Window' menu. */
         extern const char* GUI_RestrictedRuntimeWindowMenuActions;
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
         /** Holds restricted Runtime UI action types for Help menu. */
         extern const char* GUI_RestrictedRuntimeHelpMenuActions;
         /** Holds restricted Runtime UI visual-states. */
@@ -182,10 +187,10 @@ namespace UIExtraDataDefs
         extern const char* GUI_Seamless;
         /** Holds whether scaled visual-state is requested. */
         extern const char* GUI_Scale;
-#ifdef Q_WS_X11
+#ifdef VBOX_WS_X11
         /** Holds whether legacy full-screen mode is requested. */
         extern const char* GUI_Fullscreen_LegacyMode;
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
         /** Holds whether guest-screen auto-resize according machine-window size is enabled. */
         extern const char* GUI_AutoresizeGuest;
         /** Prefix used by composite extra-data keys,
@@ -215,26 +220,28 @@ namespace UIExtraDataDefs
         extern const char* GUI_HiDPI_UnscaledOutput;
         /** Holds Runtime UI HiDPI optimization type. */
         extern const char* GUI_HiDPI_Optimization;
-#ifndef Q_WS_MAC
+#ifndef VBOX_WS_MAC
         /** Holds whether mini-toolbar is enabled for full and seamless screens. */
         extern const char* GUI_ShowMiniToolBar;
         /** Holds whether mini-toolbar should auto-hide itself. */
         extern const char* GUI_MiniToolBarAutoHide;
         /** Holds mini-toolbar alignment. */
         extern const char* GUI_MiniToolBarAlignment;
-#endif /* !Q_WS_MAC */
+#endif /* !VBOX_WS_MAC */
         /** Holds Runtime UI status-bar availability status. */
         extern const char* GUI_StatusBar_Enabled;
         /** Holds restricted Runtime UI status-bar indicators. */
         extern const char* GUI_RestrictedStatusBarIndicators;
         /** Holds Runtime UI status-bar indicator order. */
         extern const char* GUI_StatusBar_IndicatorOrder;
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
         /** Mac OS X: Holds whether Dock icon should be updated at runtime. */
         extern const char* GUI_RealtimeDockIconUpdateEnabled;
         /** Mac OS X: Holds guest-screen which Dock icon should reflect at runtime. */
         extern const char* GUI_RealtimeDockIconUpdateMonitor;
-#endif /* Q_WS_MAC */
+        /** Mac OS X: Holds whether Dock icon should have overlay disabled. */
+        extern const char* GUI_DockIconDisableOverlay;
+#endif /* VBOX_WS_MAC */
         /** Holds whether machine should pass CAD to guest. */
         extern const char* GUI_PassCAD;
         /** Holds the mouse capture policy. */
@@ -253,6 +260,8 @@ namespace UIExtraDataDefs
       * @{ */
         /** Holds information-window geometry. */
         extern const char* GUI_InformationWindowGeometry;
+        /** Holds information-window elements. */
+        extern const char* GUI_InformationWindowElements;
     /** @} */
 
     /** @name Virtual Machine: Close dialog
@@ -277,7 +286,7 @@ namespace UIExtraDataDefs
     /** @} */
 #endif /* VBOX_WITH_DEBUGGER_GUI */
 
-#ifdef DEBUG
+#ifdef VBOX_GUI_WITH_EXTRADATA_MANAGER_UI
     /** @name VirtualBox: Extra-data Manager window
       * @{ */
         /** Holds extra-data manager geometry. */
@@ -285,7 +294,7 @@ namespace UIExtraDataDefs
         /** Holds extra-data manager splitter hints. */
         extern const char* GUI_ExtraDataManager_SplitterHints;
     /** @} */
-#endif /* DEBUG */
+#endif /* VBOX_GUI_WITH_EXTRADATA_MANAGER_UI */
 
     /** @name Virtual Machine: Log dialog
       * @{ */
@@ -356,8 +365,11 @@ public:
         MenuHelpActionType_Invalid              = 0,
         MenuHelpActionType_Contents             = RT_BIT(0),
         MenuHelpActionType_WebSite              = RT_BIT(1),
+        MenuHelpActionType_BugTracker           = RT_BIT(2),
+        MenuHelpActionType_Forums               = RT_BIT(3),
+        MenuHelpActionType_Oracle               = RT_BIT(4),
 #ifndef RT_OS_DARWIN
-        MenuHelpActionType_About                = RT_BIT(2),
+        MenuHelpActionType_About                = RT_BIT(5),
 #endif /* !RT_OS_DARWIN */
         MenuHelpActionType_All                  = 0xFFFF
     };
@@ -371,10 +383,11 @@ public:
         RuntimeMenuMachineActionType_InformationDialog = RT_BIT(2),
         RuntimeMenuMachineActionType_Pause             = RT_BIT(3),
         RuntimeMenuMachineActionType_Reset             = RT_BIT(4),
-        RuntimeMenuMachineActionType_SaveState         = RT_BIT(5),
-        RuntimeMenuMachineActionType_Shutdown          = RT_BIT(6),
-        RuntimeMenuMachineActionType_PowerOff          = RT_BIT(7),
-        RuntimeMenuMachineActionType_Nothing           = RT_BIT(8),
+        RuntimeMenuMachineActionType_Detach            = RT_BIT(5),
+        RuntimeMenuMachineActionType_SaveState         = RT_BIT(6),
+        RuntimeMenuMachineActionType_Shutdown          = RT_BIT(7),
+        RuntimeMenuMachineActionType_PowerOff          = RT_BIT(8),
+        RuntimeMenuMachineActionType_Nothing           = RT_BIT(9),
         RuntimeMenuMachineActionType_All               = 0xFFFF
     };
 
@@ -412,18 +425,20 @@ public:
     /** Runtime UI: Menu "Input": Action types. */
     enum RuntimeMenuInputActionType
     {
-        RuntimeMenuInputActionType_Invalid           = 0,
-        RuntimeMenuInputActionType_Keyboard          = RT_BIT(0),
-        RuntimeMenuInputActionType_KeyboardSettings  = RT_BIT(1),
-        RuntimeMenuInputActionType_TypeCAD           = RT_BIT(2),
-#ifdef Q_WS_X11
-        RuntimeMenuInputActionType_TypeCABS          = RT_BIT(3),
-#endif /* Q_WS_X11 */
-        RuntimeMenuInputActionType_TypeCtrlBreak     = RT_BIT(4),
-        RuntimeMenuInputActionType_TypeInsert        = RT_BIT(5),
-        RuntimeMenuInputActionType_Mouse             = RT_BIT(6),
-        RuntimeMenuInputActionType_MouseIntegration  = RT_BIT(7),
-        RuntimeMenuInputActionType_All               = 0xFFFF
+        RuntimeMenuInputActionType_Invalid            = 0,
+        RuntimeMenuInputActionType_Keyboard           = RT_BIT(0),
+        RuntimeMenuInputActionType_KeyboardSettings   = RT_BIT(1),
+        RuntimeMenuInputActionType_TypeCAD            = RT_BIT(2),
+#ifdef VBOX_WS_X11
+        RuntimeMenuInputActionType_TypeCABS           = RT_BIT(3),
+#endif /* VBOX_WS_X11 */
+        RuntimeMenuInputActionType_TypeCtrlBreak      = RT_BIT(4),
+        RuntimeMenuInputActionType_TypeInsert         = RT_BIT(5),
+        RuntimeMenuInputActionType_TypePrintScreen    = RT_BIT(6),
+        RuntimeMenuInputActionType_TypeAltPrintScreen = RT_BIT(7),
+        RuntimeMenuInputActionType_Mouse              = RT_BIT(8),
+        RuntimeMenuInputActionType_MouseIntegration   = RT_BIT(9),
+        RuntimeMenuInputActionType_All                = 0xFFFF
     };
 
     /** Runtime UI: Menu "Devices": Action types. */
@@ -473,6 +488,13 @@ public:
 #endif /* RT_OS_DARWIN */
 };
 
+/** Common UI: Event handling types. */
+enum EventHandlingType
+{
+    EventHandlingType_Active,
+    EventHandlingType_Passive
+};
+
 /** Common UI: Global settings page types. */
 enum GlobalSettingsPageType
 {
@@ -645,14 +667,39 @@ enum HiDPIOptimizationType
     HiDPIOptimizationType_Performance
 };
 
-#ifndef Q_WS_MAC
+#ifndef VBOX_WS_MAC
 /** Runtime UI: Mini-toolbar alignment. */
 enum MiniToolbarAlignment
 {
     MiniToolbarAlignment_Bottom,
     MiniToolbarAlignment_Top
 };
-#endif /* !Q_WS_MAC */
+#endif /* !VBOX_WS_MAC */
+
+/** Runtime UI: Information-element types. */
+enum InformationElementType
+{
+    InformationElementType_Invalid,
+    InformationElementType_General,
+    InformationElementType_System,
+    InformationElementType_Preview,
+    InformationElementType_Display,
+    InformationElementType_Storage,
+    InformationElementType_Audio,
+    InformationElementType_Network,
+    InformationElementType_Serial,
+#ifdef VBOX_WITH_PARALLEL_PORTS
+    InformationElementType_Parallel,
+#endif /* VBOX_WITH_PARALLEL_PORTS */
+    InformationElementType_USB,
+    InformationElementType_SharedFolders,
+    InformationElementType_UI,
+    InformationElementType_Description,
+    InformationElementType_RuntimeAttributes,
+    InformationElementType_StorageStatistics,
+    InformationElementType_NetworkStatistics
+};
+Q_DECLARE_METATYPE(InformationElementType);
 
 #endif /* !___UIExtraDataDefs_h___ */
 
