@@ -1,12 +1,10 @@
 /* $Id: UIGChooserItemGroup.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UIGChooserItemGroup class implementation
+ * VBox Qt GUI - UIGChooserItemGroup class implementation.
  */
 
 /*
- * Copyright (C) 2012 Oracle Corporation
+ * Copyright (C) 2012-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,23 +15,31 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* Qt includes: */
-#include <QPainter>
-#include <QStyleOptionGraphicsItem>
-#include <QGraphicsSceneDragDropEvent>
-#include <QLineEdit>
-#include <QGraphicsProxyWidget>
-#include <QGraphicsScene>
-#include <QHBoxLayout>
-#include <QMenu>
+# include <QPainter>
+# include <QStyleOptionGraphicsItem>
+# include <QGraphicsSceneDragDropEvent>
+# include <QLineEdit>
+# include <QGraphicsScene>
+# include <QHBoxLayout>
+# include <QMenu>
 
 /* GUI includes: */
-#include "UIGChooserItemGroup.h"
-#include "UIGChooserItemMachine.h"
-#include "UIGChooserModel.h"
-#include "UIIconPool.h"
-#include "UIGraphicsRotatorButton.h"
-#include "UIGChooserView.h"
+# include "UIGChooserItemGroup.h"
+# include "UIGChooserItemMachine.h"
+# include "UIGChooserModel.h"
+# include "UIIconPool.h"
+# include "UIGraphicsRotatorButton.h"
+# include "UIGChooserView.h"
+
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
+#include <QGraphicsProxyWidget>
+
 
 /* static */
 QString UIGChooserItemGroup::className() { return "UIGChooserItemGroup"; }
@@ -58,11 +64,6 @@ UIGChooserItemGroup::UIGChooserItemGroup(QGraphicsScene *pScene)
             model(), SIGNAL(sigRootItemMinimumWidthHintChanged(int)));
     connect(this, SIGNAL(sigMinimumHeightHintChanged(int)),
             model(), SIGNAL(sigRootItemMinimumHeightHintChanged(int)));
-
-    /* Init: */
-    updateVisibleName();
-    updateItemCountInfo();
-    updateToolTip();
 }
 
 UIGChooserItemGroup::UIGChooserItemGroup(QGraphicsScene *pScene,
@@ -87,8 +88,8 @@ UIGChooserItemGroup::UIGChooserItemGroup(QGraphicsScene *pScene,
     retranslateUi();
 
     /* Init: */
-    updateVisibleName();
     updateItemCountInfo();
+    updateVisibleName();
     updateToolTip();
 }
 
@@ -115,9 +116,15 @@ UIGChooserItemGroup::UIGChooserItemGroup(UIGChooserItem *pParent,
     retranslateUi();
 
     /* Init: */
-    updateVisibleName();
     updateItemCountInfo();
+    updateVisibleName();
     updateToolTip();
+
+    /* Prepare root-item connections: */
+    connect(this, SIGNAL(sigMinimumWidthHintChanged(int)),
+            model(), SIGNAL(sigRootItemMinimumWidthHintChanged(int)));
+    connect(this, SIGNAL(sigMinimumHeightHintChanged(int)),
+            model(), SIGNAL(sigRootItemMinimumHeightHintChanged(int)));
 }
 
 UIGChooserItemGroup::UIGChooserItemGroup(UIGChooserItem *pParent,
@@ -145,8 +152,8 @@ UIGChooserItemGroup::UIGChooserItemGroup(UIGChooserItem *pParent,
     retranslateUi();
 
     /* Init: */
-    updateVisibleName();
     updateItemCountInfo();
+    updateVisibleName();
     updateToolTip();
 }
 
@@ -212,9 +219,8 @@ void UIGChooserItemGroup::setName(const QString &strName)
     /* Remember new name: */
     m_strName = strName;
 
-    /* Update visible name: */
+    /* Update linked values: */
     updateVisibleName();
-    /* Update minimum header size: */
     updateMinimumHeaderSize();
 }
 
@@ -398,8 +404,8 @@ void UIGChooserItemGroup::prepare()
     m_nameFont = font();
     m_nameFont.setWeight(QFont::Bold);
     m_infoFont = font();
-    m_groupsPixmap = QPixmap(":/nw_16px.png");
-    m_machinesPixmap = QPixmap(":/machine_16px.png");
+    m_groupsPixmap = QPixmap(":/group_abstract_16px.png");
+    m_machinesPixmap = QPixmap(":/machine_abstract_16px.png");
     m_pixmapSizeGroups = m_groupsPixmap.size();
     m_pixmapSizeMachines = m_machinesPixmap.size();
     m_minimumHeaderSize = QSize(0, 0);
@@ -414,7 +420,7 @@ void UIGChooserItemGroup::prepare()
         m_pToggleButton->hide();
 
         /* Setup enter-button: */
-        m_pEnterButton = new UIGraphicsButton(this, UIGraphicsButtonType_DirectArrow);
+        m_pEnterButton = new UIGraphicsButton(this, UIIconPool::iconSet(":/next_16px.png"));
         connect(m_pEnterButton, SIGNAL(sigButtonClicked()), this, SLOT(sltIndentRoot()));
         m_pEnterButton->hide();
 
@@ -430,11 +436,10 @@ void UIGChooserItemGroup::prepare()
     if (!isMainRoot())
     {
         /* Setup exit-button: */
-        m_pExitButton = new UIGraphicsButton(this, UIGraphicsButtonType_DirectArrow);
+        m_pExitButton = new UIGraphicsButton(this, UIIconPool::iconSet(":/previous_16px.png"));
         connect(m_pExitButton, SIGNAL(sigButtonClicked()), this, SLOT(sltUnindentRoot()));
         QSizeF sh = m_pExitButton->minimumSizeHint();
         m_pExitButton->setTransformOriginPoint(sh.width() / 2, sh.height() / 2);
-        m_pExitButton->setRotation(180);
         m_pExitButton->hide();
     }
 
@@ -457,9 +462,11 @@ void UIGChooserItemGroup::copyContent(UIGChooserItemGroup *pFrom, UIGChooserItem
 
 void UIGChooserItemGroup::handleRootStatusChange()
 {
-    /* Update visible name: */
+    /* Call to base-class: */
+    UIGChooserItem::handleRootStatusChange();
+
+    /* Update linked values: */
     updateVisibleName();
-    /* Update minimum header size: */
     updateMinimumHeaderSize();
 }
 
@@ -481,7 +488,7 @@ void UIGChooserItemGroup::updateVisibleName()
     int iMachinePixmapWidth = m_pixmapSizeMachines.width();
     int iGroupCountTextWidth = m_infoSizeGroups.width();
     int iMachineCountTextWidth = m_infoSizeMachines.width();
-    int iMaximumWidth = geometry().width();
+    int iMaximumWidth = (int)geometry().width();
 
     /* Left margin: */
     if (isRoot())
@@ -512,27 +519,64 @@ void UIGChooserItemGroup::updateVisibleName()
     iMaximumWidth -= iHorizontalMargin;
     if (isRoot())
         iMaximumWidth -= iRootIndent;
-
-    /* Recache visible name: */
+    /* Calculate new visible name and name-size: */
     QPaintDevice *pPaintDevice = model()->paintDevice();
-    m_strVisibleName = isMainRoot() ? QString() : compressText(m_nameFont, pPaintDevice, name(), iMaximumWidth);
-    m_visibleNameSize = isMainRoot() ? QSize(0, 0) : textSize(m_nameFont, pPaintDevice, m_strVisibleName);
+    QString strVisibleName = compressText(m_nameFont, pPaintDevice, name(), iMaximumWidth);
+    QSize visibleNameSize = textSize(m_nameFont, pPaintDevice, strVisibleName);
 
-    /* Repaint item: */
-    update();
+    /* Update linked values: */
+    if (m_visibleNameSize != visibleNameSize)
+    {
+        m_visibleNameSize = visibleNameSize;
+        updateGeometry();
+    }
+    if (m_strVisibleName != strVisibleName)
+    {
+        m_strVisibleName = strVisibleName;
+        update();
+    }
 }
 
 void UIGChooserItemGroup::updateItemCountInfo()
 {
+    /* Not for main root: */
+    if (isMainRoot())
+        return;
+
     /* Update item info attributes: */
     QPaintDevice *pPaintDevice = model()->paintDevice();
-    m_strInfoGroups = isMainRoot() || m_groupItems.isEmpty() ? QString() : QString::number(m_groupItems.size());
-    m_strInfoMachines = isMainRoot() || m_machineItems.isEmpty() ? QString() : QString::number(m_machineItems.size());
-    m_infoSizeGroups = isMainRoot() ? QSize(0, 0) : textSize(m_infoFont, pPaintDevice, m_strInfoGroups);
-    m_infoSizeMachines = isMainRoot() ? QSize(0, 0) : textSize(m_infoFont, pPaintDevice, m_strInfoMachines);
+    QString strInfoGroups = m_groupItems.isEmpty() ? QString() : QString::number(m_groupItems.size());
+    QString strInfoMachines = m_machineItems.isEmpty() ? QString() : QString::number(m_machineItems.size());
+    QSize infoSizeGroups = textSize(m_infoFont, pPaintDevice, strInfoGroups);
+    QSize infoSizeMachines = textSize(m_infoFont, pPaintDevice, strInfoMachines);
 
     /* Update linked values: */
-    updateMinimumHeaderSize();
+    bool fSomethingChanged = false;
+    if (m_strInfoGroups != strInfoGroups)
+    {
+        m_strInfoGroups = strInfoGroups;
+        fSomethingChanged = true;
+    }
+    if (m_strInfoMachines != strInfoMachines)
+    {
+        m_strInfoMachines = strInfoMachines;
+        fSomethingChanged = true;
+    }
+    if (m_infoSizeGroups != infoSizeGroups)
+    {
+        m_infoSizeGroups = infoSizeGroups;
+        fSomethingChanged = true;
+    }
+    if (m_infoSizeMachines != infoSizeMachines)
+    {
+        m_infoSizeMachines = infoSizeMachines;
+        fSomethingChanged = true;
+    }
+    if (fSomethingChanged)
+    {
+        updateVisibleName();
+        updateMinimumHeaderSize();
+    }
 }
 
 void UIGChooserItemGroup::updateMinimumHeaderSize()
@@ -576,7 +620,7 @@ void UIGChooserItemGroup::updateMinimumHeaderSize()
     if (!isRoot())
         iHeaderWidth += (iMinorSpacing + m_enterButtonSize.width());
 
-    /* Search for maximum height: */
+    /* Calculate maximum height: */
     QList<int> heights;
     /* Button height: */
     if (isRoot())
@@ -596,10 +640,15 @@ void UIGChooserItemGroup::updateMinimumHeaderSize()
     foreach (int iHeight, heights)
         iHeaderHeight = qMax(iHeaderHeight, iHeight);
 
-    /* Recache minimum header size: */
-    m_minimumHeaderSize = QSize(iHeaderWidth, iHeaderHeight);
+    /* Calculate new minimum header size: */
+    QSize minimumHeaderSize = QSize(iHeaderWidth, iHeaderHeight);
+
+    /* Is there something changed? */
+    if (m_minimumHeaderSize == minimumHeaderSize)
+        return;
 
     /* Update linked values: */
+    m_minimumHeaderSize = minimumHeaderSize;
     updateGeometry();
 }
 
@@ -746,10 +795,10 @@ void UIGChooserItemGroup::addItem(UIGChooserItem *pItem, int iPosition)
         }
     }
 
-    /* Update: */
-    updateVisibleName();
+    /* Update linked values: */
     updateItemCountInfo();
     updateToolTip();
+    updateGeometry();
 }
 
 void UIGChooserItemGroup::removeItem(UIGChooserItem *pItem)
@@ -778,10 +827,10 @@ void UIGChooserItemGroup::removeItem(UIGChooserItem *pItem)
         }
     }
 
-    /* Update: */
-    updateVisibleName();
+    /* Update linked values: */
     updateItemCountInfo();
     updateToolTip();
+    updateGeometry();
 }
 
 void UIGChooserItemGroup::setItems(const QList<UIGChooserItem*> &items, UIGChooserItemType type)
@@ -794,10 +843,10 @@ void UIGChooserItemGroup::setItems(const QList<UIGChooserItem*> &items, UIGChoos
         default: AssertMsgFailed(("Invalid item type!")); break;
     }
 
-    /* Update: */
-    updateVisibleName();
+    /* Update linked values: */
     updateItemCountInfo();
     updateToolTip();
+    updateGeometry();
 }
 
 QList<UIGChooserItem*> UIGChooserItemGroup::items(UIGChooserItemType type /* = UIGChooserItemType_Any */) const
@@ -850,10 +899,10 @@ void UIGChooserItemGroup::clearItems(UIGChooserItemType type /* = UIGChooserItem
         }
     }
 
-    /* Update: */
-    updateVisibleName();
+    /* Update linked values: */
     updateItemCountInfo();
     updateToolTip();
+    updateGeometry();
 }
 
 void UIGChooserItemGroup::updateAll(const QString &strId)
@@ -1007,7 +1056,7 @@ void UIGChooserItemGroup::updateLayout()
         if (m_pEnterButton)
         {
             /* Prepare variables: */
-            int iFullWidth = geometry().width();
+            int iFullWidth = (int)geometry().width();
             int iEnterButtonWidth = m_enterButtonSize.width();
             int iEnterButtonHeight = m_enterButtonSize.height();
             /* Layout enter-button: */
@@ -1027,7 +1076,7 @@ void UIGChooserItemGroup::updateLayout()
             int iNameEditorX = iHorizontalMargin + iToggleButtonWidth + iMajorSpacing;
             int iNameEditorY = 1;
             m_pNameEditor->setPos(iNameEditorX, iNameEditorY);
-            m_pNameEditorWidget->resize(geometry().width() - iNameEditorX - iHorizontalMargin, m_pNameEditorWidget->height());
+            m_pNameEditorWidget->resize((int)(geometry().width() - iNameEditorX - iHorizontalMargin), m_pNameEditorWidget->height());
         }
 
         /* Prepare body indent: */
@@ -1288,6 +1337,7 @@ bool UIGChooserItemGroup::isDropAllowed(QGraphicsSceneDragDropEvent *pEvent, Dra
                 /* Make sure there is no other item with such id: */
                 return !isContainsMachine(pItem->toMachineItem()->id());
             }
+            default: break; /* Shut up, MSC! */
         }
     }
     /* That was invalid mime: */
@@ -1459,7 +1509,7 @@ void UIGChooserItemGroup::hoverMoveEvent(QGraphicsSceneHoverEvent *pEvent)
     /* Call to base-class: */
     UIGChooserItem::hoverMoveEvent(pEvent);
 
-    /* Update visible name: */
+    /* Update linked values: */
     updateVisibleName();
 }
 
@@ -1472,7 +1522,7 @@ void UIGChooserItemGroup::hoverLeaveEvent(QGraphicsSceneHoverEvent *pEvent)
     /* Call to base-class: */
     UIGChooserItem::hoverLeaveEvent(pEvent);
 
-    /* Update visible name: */
+    /* Update linked values: */
     updateVisibleName();
 }
 
@@ -1700,8 +1750,8 @@ void UIGChooserItemGroup::paintHeader(QPainter *pPainter, const QRect &rect)
                                   iVerticalMargin : iVerticalMargin + (iFullHeaderHeight - m_pixmapSizeMachines.height()) / 2;
             paintPixmap(/* Painter: */
                         pPainter,
-                        /* Rectangle to paint in: */
-                        QRect(QPoint(iMachinePixmapX, iMachinePixmapY), m_pixmapSizeMachines),
+                        /* Point to paint in: */
+                        QPoint(iMachinePixmapX, iMachinePixmapY),
                         /* Pixmap to paint: */
                         m_machinesPixmap);
         }
@@ -1730,8 +1780,8 @@ void UIGChooserItemGroup::paintHeader(QPainter *pPainter, const QRect &rect)
                                 iVerticalMargin : iVerticalMargin + (iFullHeaderHeight - m_pixmapSizeGroups.height()) / 2;
             paintPixmap(/* Painter: */
                         pPainter,
-                        /* Rectangle to paint in: */
-                        QRect(QPoint(iGroupPixmapX, iGroupPixmapY), m_pixmapSizeGroups),
+                        /* Point to paint in: */
+                        QPoint(iGroupPixmapX, iGroupPixmapY),
                         /* Pixmap to paint: */
                         m_groupsPixmap);
         }
@@ -1753,7 +1803,7 @@ void UIGChooserItemGroup::updateAnimationParameters()
     /* Recalculate animation parameters: */
     QSizeF openedSize = minimumSizeHint(true);
     QSizeF closedSize = minimumSizeHint(false);
-    int iAdditionalHeight = openedSize.height() - closedSize.height();
+    int iAdditionalHeight = (int)(openedSize.height() - closedSize.height());
     m_pToggleButton->setAnimationRange(0, iAdditionalHeight);
 }
 
