@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2015 Oracle Corporation
+ * Copyright (C) 2010-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -66,7 +66,13 @@ static void test1Worker(RTTEST hTest, const char *pszBaseDir,
     /* Create it.*/
     RTTESTI_CHECK_RC_OK_RETV(RTPathJoin(szPath1, sizeof(szPath1), pszBaseDir, "tstRTSymlink-link-1"));
     RTSymlinkDelete(szPath1, 0); /* clean up previous run */
-    RTTESTI_CHECK_RC_RETV(RTSymlinkCreate(szPath1, pszTarget, RTSYMLINKTYPE_FILE, 0), VINF_SUCCESS);
+    int rc = RTSymlinkCreate(szPath1, pszTarget, RTSYMLINKTYPE_FILE, 0);
+    if (rc == VERR_NOT_SUPPORTED)
+    {
+        RTTestPrintf(hTest, RTTESTLVL_ALWAYS, "VERR_NOT_SUPPORTED - skipping\n");
+        return;
+    }
+    RTTESTI_CHECK_RC_RETV(rc, VINF_SUCCESS);
 
     /* Check the predicate functions. */
     RTTESTI_CHECK(RTSymlinkExists(szPath1));
@@ -91,7 +97,6 @@ static void test1Worker(RTTEST hTest, const char *pszBaseDir,
                       ("got=\"%s\" expected=\"%.*s\"", szPath2, cchTarget - 1, szPath3));
 
     /* Other APIs that have to handle symlinks carefully. */
-    int rc;
     RTFSOBJINFO ObjInfo;
     RTTESTI_CHECK_RC(rc = RTPathQueryInfo(szPath1, &ObjInfo, RTFSOBJATTRADD_NOTHING), VINF_SUCCESS);
     if (RT_SUCCESS(rc))
@@ -131,7 +136,6 @@ static void test1Worker(RTTEST hTest, const char *pszBaseDir,
 static void test1(RTTEST hTest, const char *pszBaseDir)
 {
     char szPath1[RTPATH_MAX];
-    char szPath2[RTPATH_MAX];
 
     /*
      * Making some assumptions about how we are executed from to start with...
@@ -139,12 +143,10 @@ static void test1(RTTEST hTest, const char *pszBaseDir)
     RTTestISub("Negative RTSymlinkRead, Exists & IsDangling");
     char szExecDir[RTPATH_MAX];
     RTTESTI_CHECK_RC_OK_RETV(RTPathExecDir(szExecDir, sizeof(szExecDir)));
-    size_t cchExecDir = strlen(szExecDir);
     RTTESTI_CHECK(RTDirExists(szExecDir));
 
     char szExecFile[RTPATH_MAX];
     RTTESTI_CHECK_RETV(RTProcGetExecutablePath(szExecFile, sizeof(szExecFile)) != NULL);
-    size_t cchExecFile = strlen(szExecFile);
     RTTESTI_CHECK(RTFileExists(szExecFile));
 
     RTTESTI_CHECK(!RTSymlinkExists(szExecFile));
@@ -182,7 +184,7 @@ static void test1(RTTEST hTest, const char *pszBaseDir)
 }
 
 
-int main(int argc, char **argv)
+int main()
 {
     RTTEST hTest;
     int rc = RTTestInitAndCreate("tstRTSymlink", &hTest);
