@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,11 +15,12 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_VD_RAW /** @todo logging group */
-#include <VBox/vd-cache-plugin.h>
+#include <VBox/vd-cache-backend.h>
 #include <VBox/err.h>
 
 #include <VBox/log.h>
@@ -28,6 +29,8 @@
 #include <iprt/file.h>
 #include <iprt/asm.h>
 
+#include "VDBackends.h"
+
 /*******************************************************************************
 * On disk data structures                                                      *
 *******************************************************************************/
@@ -199,9 +202,10 @@ AssertCompileSize(VciBlkMap, VCI_BLOCK_SIZE);
 /** Block bitmap entry */
 typedef uint8_t VciBlkMapEnt;
 
-/*******************************************************************************
-*   Constants And Macros, Structures and Typedefs                              *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Constants And Macros, Structures and Typedefs                                                                                *
+*********************************************************************************************************************************/
 
 /**
  * Block range descriptor.
@@ -371,9 +375,10 @@ typedef struct VCICACHE
 #define VCIBLKMAP_ALLOC_META RT_BIT(0)
 #define VCIBLKMAP_ALLOC_MASK 0x1
 
-/*******************************************************************************
-*   Static Variables                                                           *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Static Variables                                                                                                             *
+*********************************************************************************************************************************/
 
 /** NULL-terminated array of supported file extensions. */
 static const char *const s_apszVciFileExtensions[] =
@@ -382,9 +387,10 @@ static const char *const s_apszVciFileExtensions[] =
     NULL
 };
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 
 /**
  * Internal. Flush image data to disk.
@@ -486,6 +492,7 @@ static int vciBlkMapCreate(uint64_t cBlocks, PVCIBLKMAP *ppBlkMap, uint32_t *pcB
     return rc;
 }
 
+#if 0 /** @todo unsued vciBlkMapDestroy */
 /**
  * Frees a block map.
  *
@@ -511,6 +518,7 @@ static void vciBlkMapDestroy(PVCIBLKMAP pBlkMap)
 
     LogFlowFunc(("returns\n"));
 }
+#endif
 
 /**
  * Loads the block map from the specified medium and creates all necessary
@@ -535,7 +543,7 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
         cBlkMap -= VCI_BYTE2BLOCK(sizeof(VciBlkMap));
 
         rc = vdIfIoIntFileReadSync(pStorage->pIfIo, pStorage->pStorage, offBlkMap,
-                                   &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)), NULL);
+                                   &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)));
         if (RT_SUCCESS(rc))
         {
             offBlkMap += VCI_BYTE2BLOCK(sizeof(VciBlkMap));
@@ -561,8 +569,6 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                     pBlkMap->cBlocksAllocData = BlkMap.cBlocksAllocData;
 
                     /* Load the bitmap and construct the range list. */
-                    uint32_t cBlocksFree = 0;
-                    uint32_t cBlocksAllocated = 0;
                     PVCIBLKRANGEDESC pRangeCur = (PVCIBLKRANGEDESC)RTMemAllocZ(sizeof(VCIBLKRANGEDESC));
 
                     if (pRangeCur)
@@ -574,7 +580,7 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                         cBlocksRead = RT_MIN(VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)), cBlocksLeft);
                         rc = vdIfIoIntFileReadSync(pStorage->pIfIo, pStorage->pStorage,
                                                    offBlkMap, abBitmapBuffer,
-                                                   cBlocksRead, NULL);
+                                                   cBlocksRead);
 
                         if (RT_SUCCESS(rc))
                         {
@@ -586,6 +592,8 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                             pBlkMap->pRangesHead = pRangeCur;
                             pBlkMap->pRangesTail = pRangeCur;
                         }
+                        else
+                            RTMemFree(pRangeCur);
 
                         while (   RT_SUCCESS(rc)
                                && cBlocksLeft)
@@ -647,7 +655,7 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                                 /* Read next chunk. */
                                 cBlocksRead = RT_MIN(VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)), cBlocksLeft);
                                 rc = vdIfIoIntFileReadSync(pStorage->pIfIo, pStorage->pStorage,
-                                                           offBlkMap, abBitmapBuffer, cBlocksRead, NULL);
+                                                           offBlkMap, abBitmapBuffer, cBlocksRead);
                             }
                         }
                     }
@@ -660,8 +668,8 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                         LogFlowFunc(("return success\n"));
                         return VINF_SUCCESS;
                     }
-                    else
-                        RTMemFree(pBlkMap);
+
+                    RTMemFree(pBlkMap);
                 }
                 else
                     rc = VERR_NO_MEMORY;
@@ -669,7 +677,7 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
             else
                 rc = VERR_VD_GEN_INVALID_HEADER;
         }
-        else if (RT_SUCCESS(rc))
+        else
             rc = VERR_VD_GEN_INVALID_HEADER;
     }
     else
@@ -711,7 +719,7 @@ static int vciBlkMapSave(PVCIBLKMAP pBlkMap, PVCICACHE pStorage, uint64_t offBlk
         BlkMap.cBlocksAllocData = RT_H2LE_U32(pBlkMap->cBlocksAllocData);
 
         rc = vdIfIoIntFileWriteSync(pStorage->pIfIo, pStorage->pStorage, offBlkMap,
-                                    &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)), NULL);
+                                    &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)));
         if (RT_SUCCESS(rc))
         {
             uint8_t abBitmapBuffer[16*_1K];
@@ -742,7 +750,7 @@ static int vciBlkMapSave(PVCIBLKMAP pBlkMap, PVCICACHE pStorage, uint64_t offBlk
                         /* Buffer is full, write to file and reset. */
                         rc = vdIfIoIntFileWriteSync(pStorage->pIfIo, pStorage->pStorage,
                                                     offBlkMap, abBitmapBuffer,
-                                                    VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)), NULL);
+                                                    VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)));
                         if (RT_FAILURE(rc))
                             break;
 
@@ -758,16 +766,17 @@ static int vciBlkMapSave(PVCIBLKMAP pBlkMap, PVCICACHE pStorage, uint64_t offBlk
 
             if (RT_SUCCESS(rc) && iBit)
                 rc = vdIfIoIntFileWriteSync(pStorage->pIfIo, pStorage->pStorage,
-                                            offBlkMap, abBitmapBuffer, VCI_BYTE2BLOCK(iBit / 8), NULL);
+                                            offBlkMap, abBitmapBuffer, VCI_BYTE2BLOCK(iBit / 8));
         }
     }
     else
-        rc = VERR_INTERNAL_ERROR; /* @todo Better error code. */
+        rc = VERR_INTERNAL_ERROR; /** @todo Better error code. */
 
     LogFlowFunc(("returns rc=%Rrc\n", rc));
     return rc;
 }
 
+#if 0 /* unused */
 /**
  * Finds the range block describing the given block address.
  *
@@ -785,6 +794,7 @@ static PVCIBLKRANGEDESC vciBlkMapFindByBlock(PVCIBLKMAP pBlkMap, uint64_t offBlo
 
     return pBlk;
 }
+#endif
 
 /**
  * Allocates the given number of blocks in the bitmap and returns the start block address.
@@ -875,6 +885,7 @@ static int vciBlkMapAllocate(PVCIBLKMAP pBlkMap, uint32_t cBlocks, uint32_t fFla
     return rc;
 }
 
+#if 0 /* unused */
 /**
  * Try to extend the space of an already allocated block.
  *
@@ -893,11 +904,14 @@ static int vciBlkMapRealloc(PVCIBLKMAP pBlkMap, uint32_t cBlocksNew, uint64_t of
                  pBlkMap, cBlocksNew, offBlockAddrOld, poffBlockAddr));
 
     AssertMsgFailed(("Implement\n"));
+    RT_NOREF4(pBlkMap, cBlocksNew, offBlockAddrOld, poffBlockAddr);
 
     LogFlowFunc(("returns rc=%Rrc offBlockAddr=%llu\n", rc, *poffBlockAddr));
     return rc;
 }
+#endif /* unused */
 
+#if 0 /* unused */
 /**
  * Frees a range of blocks.
  *
@@ -979,6 +993,7 @@ static void vciBlkMapFree(PVCIBLKMAP pBlkMap, uint64_t offBlockAddr, uint32_t cB
 
     LogFlowFunc(("returns\n"));
 }
+#endif /* unused */
 
 /**
  * Converts a tree node from the image to the in memory structure.
@@ -1109,7 +1124,7 @@ static PVCICACHEEXTENT vciCacheExtentLookup(PVCICACHE pCache, uint64_t offBlockO
                     /* Read from disk and add to the tree. */
                     rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage,
                                                VCI_BLOCK2BYTE(pInt->PtrChild.u.offAddrBlockNode),
-                                               &NodeTree, sizeof(NodeTree), NULL);
+                                               &NodeTree, sizeof(NodeTree));
                     AssertRC(rc);
 
                     pNodeNew = vciTreeNodeImage2Host(pInt->PtrChild.u.offAddrBlockNode, &NodeTree);
@@ -1182,7 +1197,7 @@ static PVCICACHEEXTENT vciCacheExtentLookup(PVCICACHE pCache, uint64_t offBlockO
 
                 while (pInt)
                 {
-                    
+
                 }
             }
         }
@@ -1228,7 +1243,7 @@ static int vciOpenImage(PVCICACHE pCache, unsigned uOpenFlags)
     }
 
     rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage, 0, &Hdr,
-                               VCI_BYTE2BLOCK(sizeof(Hdr)), NULL);
+                               VCI_BYTE2BLOCK(sizeof(Hdr)));
     if (RT_FAILURE(rc))
     {
         rc = VERR_VD_GEN_INVALID_HEADER;
@@ -1258,7 +1273,7 @@ static int vciOpenImage(PVCICACHE pCache, unsigned uOpenFlags)
 
             rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage,
                                        pCache->offTreeRoot, &RootNode,
-                                       VCI_BYTE2BLOCK(sizeof(VciTreeNode)), NULL);
+                                       VCI_BYTE2BLOCK(sizeof(VciTreeNode)));
             if (RT_SUCCESS(rc))
             {
                 pCache->pRoot = vciTreeNodeImage2Host(pCache->offTreeRoot, &RootNode);
@@ -1285,6 +1300,7 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
                           void *pvUser, unsigned uPercentStart,
                           unsigned uPercentSpan)
 {
+    RT_NOREF1(pszComment);
     VciHdr Hdr;
     VciTreeNode NodeRoot;
     int rc;
@@ -1393,7 +1409,7 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
         Hdr.cBlkMap          = RT_H2LE_U32(cBlkMap);
 
         rc = vdIfIoIntFileWriteSync(pCache->pIfIo, pCache->pStorage, offHdr, &Hdr,
-                                    VCI_BYTE2BLOCK(sizeof(VciHdr)), NULL);
+                                    VCI_BYTE2BLOCK(sizeof(VciHdr)));
         if (RT_FAILURE(rc))
         {
             rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot write header '%s'"), pCache->pszFilename);
@@ -1412,7 +1428,7 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
         NodeRoot.u8Type = RT_H2LE_U32(VCI_TREE_NODE_TYPE_LEAF);
 
         rc = vdIfIoIntFileWriteSync(pCache->pIfIo, pCache->pStorage, offTreeRoot,
-                                    &NodeRoot, VCI_BYTE2BLOCK(sizeof(VciTreeNode)), NULL);
+                                    &NodeRoot, VCI_BYTE2BLOCK(sizeof(VciTreeNode)));
         if (RT_FAILURE(rc))
         {
             rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot write root node '%s'"), pCache->pszFilename);
@@ -1439,9 +1455,10 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
 }
 
 /** @copydoc VDCACHEBACKEND::pfnProbe */
-static int vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
-                    PVDINTERFACE pVDIfsImage)
+static DECLCALLBACK(int) vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
+                                  PVDINTERFACE pVDIfsImage)
 {
+    RT_NOREF1(pVDIfsCache);
     VciHdr Hdr;
     PVDIOSTORAGE pStorage = NULL;
     uint64_t cbFile;
@@ -1466,7 +1483,7 @@ static int vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
         goto out;
     }
 
-    rc = vdIfIoIntFileReadSync(pIfIo, pStorage, 0, &Hdr, sizeof(Hdr), NULL);
+    rc = vdIfIoIntFileReadSync(pIfIo, pStorage, 0, &Hdr, sizeof(Hdr));
     if (RT_FAILURE(rc))
     {
         rc = VERR_VD_GEN_INVALID_HEADER;
@@ -1496,9 +1513,9 @@ out:
 }
 
 /** @copydoc VDCACHEBACKEND::pfnOpen */
-static int vciOpen(const char *pszFilename, unsigned uOpenFlags,
-                   PVDINTERFACE pVDIfsDisk, PVDINTERFACE pVDIfsImage,
-                   void **ppBackendData)
+static DECLCALLBACK(int) vciOpen(const char *pszFilename, unsigned uOpenFlags,
+                                 PVDINTERFACE pVDIfsDisk, PVDINTERFACE pVDIfsImage,
+                                 void **ppBackendData)
 {
     LogFlowFunc(("pszFilename=\"%s\" uOpenFlags=%#x pVDIfsDisk=%#p pVDIfsImage=%#p ppBackendData=%#p\n", pszFilename, uOpenFlags, pVDIfsDisk, pVDIfsImage, ppBackendData));
     int rc;
@@ -1543,13 +1560,14 @@ out:
 }
 
 /** @copydoc VDCACHEBACKEND::pfnCreate */
-static int vciCreate(const char *pszFilename, uint64_t cbSize,
-                     unsigned uImageFlags, const char *pszComment,
-                     PCRTUUID pUuid, unsigned uOpenFlags,
-                     unsigned uPercentStart, unsigned uPercentSpan,
-                     PVDINTERFACE pVDIfsDisk, PVDINTERFACE pVDIfsImage,
-                     PVDINTERFACE pVDIfsOperation, void **ppBackendData)
+static DECLCALLBACK(int) vciCreate(const char *pszFilename, uint64_t cbSize,
+                                   unsigned uImageFlags, const char *pszComment,
+                                   PCRTUUID pUuid, unsigned uOpenFlags,
+                                   unsigned uPercentStart, unsigned uPercentSpan,
+                                   PVDINTERFACE pVDIfsDisk, PVDINTERFACE pVDIfsImage,
+                                   PVDINTERFACE pVDIfsOperation, void **ppBackendData)
 {
+    RT_NOREF1(pUuid);
     LogFlowFunc(("pszFilename=\"%s\" cbSize=%llu uImageFlags=%#x pszComment=\"%s\" Uuid=%RTuuid uOpenFlags=%#x uPercentStart=%u uPercentSpan=%u pVDIfsDisk=%#p pVDIfsImage=%#p pVDIfsOperation=%#p ppBackendData=%#p",
                  pszFilename, cbSize, uImageFlags, pszComment, pUuid, uOpenFlags, uPercentStart, uPercentSpan, pVDIfsDisk, pVDIfsImage, pVDIfsOperation, ppBackendData));
     int rc;
@@ -1617,7 +1635,7 @@ out:
 }
 
 /** @copydoc VDCACHEBACKEND::pfnClose */
-static int vciClose(void *pBackendData, bool fDelete)
+static DECLCALLBACK(int) vciClose(void *pBackendData, bool fDelete)
 {
     LogFlowFunc(("pBackendData=%#p fDelete=%d\n", pBackendData, fDelete));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1631,10 +1649,11 @@ static int vciClose(void *pBackendData, bool fDelete)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnRead */
-static int vciRead(void *pBackendData, uint64_t uOffset, void *pvBuf,
-                   size_t cbToRead, size_t *pcbActuallyRead)
+static DECLCALLBACK(int) vciRead(void *pBackendData, uint64_t uOffset, size_t cbToRead,
+                                 PVDIOCTX pIoCtx, size_t *pcbActuallyRead)
 {
-    LogFlowFunc(("pBackendData=%#p uOffset=%llu pvBuf=%#p cbToRead=%zu pcbActuallyRead=%#p\n", pBackendData, uOffset, pvBuf, cbToRead, pcbActuallyRead));
+    LogFlowFunc(("pBackendData=%#p uOffset=%llu cbToRead=%zu pIoCtx=%#p pcbActuallyRead=%#p\n",
+                 pBackendData, uOffset, cbToRead, pIoCtx, pcbActuallyRead));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc = VINF_SUCCESS;
     PVCICACHEEXTENT pExtent;
@@ -1651,9 +1670,9 @@ static int vciRead(void *pBackendData, uint64_t uOffset, void *pvBuf,
         uint64_t offRead = offBlockAddr - pExtent->u64BlockOffset;
         cBlocksToRead = RT_MIN(cBlocksToRead, pExtent->u32Blocks - offRead);
 
-        rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage,
+        rc = vdIfIoIntFileReadUser(pCache->pIfIo, pCache->pStorage,
                                    pExtent->u64BlockAddr + offRead,
-                                   pvBuf, cBlocksToRead, NULL);
+                                   pIoCtx, cBlocksToRead);
     }
     else
     {
@@ -1665,52 +1684,50 @@ static int vciRead(void *pBackendData, uint64_t uOffset, void *pvBuf,
     if (pcbActuallyRead)
         *pcbActuallyRead = VCI_BLOCK2BYTE(cBlocksToRead);
 
-out:
     LogFlowFunc(("returns %Rrc\n", rc));
     return rc;
 }
 
 /** @copydoc VDCACHEBACKEND::pfnWrite */
-static int vciWrite(void *pBackendData, uint64_t uOffset, const void *pvBuf,
-                    size_t cbToWrite, size_t *pcbWriteProcess)
+static DECLCALLBACK(int) vciWrite(void *pBackendData, uint64_t uOffset, size_t cbToWrite,
+                                  PVDIOCTX pIoCtx, size_t *pcbWriteProcess)
 {
-    LogFlowFunc(("pBackendData=%#p uOffset=%llu pvBuf=%#p cbToWrite=%zu pcbWriteProcess=%#p\n",
-                 pBackendData, uOffset, pvBuf, cbToWrite, pcbWriteProcess));
+    RT_NOREF5(pBackendData, uOffset, cbToWrite, pIoCtx, pcbWriteProcess);
+    LogFlowFunc(("pBackendData=%#p uOffset=%llu cbToWrite=%zu pIoCtx=%#p pcbWriteProcess=%#p\n",
+                 pBackendData, uOffset, cbToWrite, pIoCtx, pcbWriteProcess));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc = VINF_SUCCESS;
     uint64_t cBlocksToWrite = VCI_BYTE2BLOCK(cbToWrite);
-    uint64_t offBlockAddr  = VCI_BYTE2BLOCK(uOffset);
-    PVCICACHEEXTENT pExtent;
+    //uint64_t offBlockAddr  = VCI_BYTE2BLOCK(uOffset);
 
-    AssertPtr(pCache);
+    AssertPtr(pCache); NOREF(pCache);
     Assert(uOffset % 512 == 0);
     Assert(cbToWrite % 512 == 0);
-
     while (cBlocksToWrite)
     {
-        
+
     }
 
-    *pcbWriteProcess = cbToWrite; /** @todo: Implement. */
-out:
+    *pcbWriteProcess = cbToWrite; /** @todo Implement. */
+
     LogFlowFunc(("returns %Rrc\n", rc));
     return rc;
 }
 
 /** @copydoc VDCACHEBACKEND::pfnFlush */
-static int vciFlush(void *pBackendData)
+static DECLCALLBACK(int) vciFlush(void *pBackendData, PVDIOCTX pIoCtx)
 {
+    RT_NOREF1(pIoCtx);
     LogFlowFunc(("pBackendData=%#p\n", pBackendData));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
-    int rc = VINF_SUCCESS;
 
-    rc = vciFlushImage(pCache);
+    int rc = vciFlushImage(pCache);
     LogFlowFunc(("returns %Rrc\n", rc));
     return rc;
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetVersion */
-static unsigned vciGetVersion(void *pBackendData)
+static DECLCALLBACK(unsigned) vciGetVersion(void *pBackendData)
 {
     LogFlowFunc(("pBackendData=%#p\n", pBackendData));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1724,7 +1741,7 @@ static unsigned vciGetVersion(void *pBackendData)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetSize */
-static uint64_t vciGetSize(void *pBackendData)
+static DECLCALLBACK(uint64_t) vciGetSize(void *pBackendData)
 {
     LogFlowFunc(("pBackendData=%#p\n", pBackendData));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1740,7 +1757,7 @@ static uint64_t vciGetSize(void *pBackendData)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetFileSize */
-static uint64_t vciGetFileSize(void *pBackendData)
+static DECLCALLBACK(uint64_t) vciGetFileSize(void *pBackendData)
 {
     LogFlowFunc(("pBackendData=%#p\n", pBackendData));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1764,7 +1781,7 @@ static uint64_t vciGetFileSize(void *pBackendData)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetImageFlags */
-static unsigned vciGetImageFlags(void *pBackendData)
+static DECLCALLBACK(unsigned) vciGetImageFlags(void *pBackendData)
 {
     LogFlowFunc(("pBackendData=%#p\n", pBackendData));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1782,7 +1799,7 @@ static unsigned vciGetImageFlags(void *pBackendData)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetOpenFlags */
-static unsigned vciGetOpenFlags(void *pBackendData)
+static DECLCALLBACK(unsigned) vciGetOpenFlags(void *pBackendData)
 {
     LogFlowFunc(("pBackendData=%#p\n", pBackendData));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1800,7 +1817,7 @@ static unsigned vciGetOpenFlags(void *pBackendData)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnSetOpenFlags */
-static int vciSetOpenFlags(void *pBackendData, unsigned uOpenFlags)
+static DECLCALLBACK(int) vciSetOpenFlags(void *pBackendData, unsigned uOpenFlags)
 {
     LogFlowFunc(("pBackendData=%#p\n uOpenFlags=%#x", pBackendData, uOpenFlags));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
@@ -1826,9 +1843,10 @@ out:
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetComment */
-static int vciGetComment(void *pBackendData, char *pszComment,
-                          size_t cbComment)
+static DECLCALLBACK(int) vciGetComment(void *pBackendData, char *pszComment,
+                                       size_t cbComment)
 {
+    RT_NOREF2(pszComment, cbComment);
     LogFlowFunc(("pBackendData=%#p pszComment=%#p cbComment=%zu\n", pBackendData, pszComment, cbComment));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc;
@@ -1845,8 +1863,9 @@ static int vciGetComment(void *pBackendData, char *pszComment,
 }
 
 /** @copydoc VDCACHEBACKEND::pfnSetComment */
-static int vciSetComment(void *pBackendData, const char *pszComment)
+static DECLCALLBACK(int) vciSetComment(void *pBackendData, const char *pszComment)
 {
+    RT_NOREF1(pszComment);
     LogFlowFunc(("pBackendData=%#p pszComment=\"%s\"\n", pBackendData, pszComment));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc;
@@ -1863,14 +1882,14 @@ static int vciSetComment(void *pBackendData, const char *pszComment)
     else
         rc = VERR_VD_NOT_OPENED;
 
-out:
     LogFlowFunc(("returns %Rrc\n", rc));
     return rc;
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetUuid */
-static int vciGetUuid(void *pBackendData, PRTUUID pUuid)
+static DECLCALLBACK(int) vciGetUuid(void *pBackendData, PRTUUID pUuid)
 {
+    RT_NOREF1(pUuid);
     LogFlowFunc(("pBackendData=%#p pUuid=%#p\n", pBackendData, pUuid));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc;
@@ -1887,8 +1906,9 @@ static int vciGetUuid(void *pBackendData, PRTUUID pUuid)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnSetUuid */
-static int vciSetUuid(void *pBackendData, PCRTUUID pUuid)
+static DECLCALLBACK(int) vciSetUuid(void *pBackendData, PCRTUUID pUuid)
 {
+    RT_NOREF1(pUuid);
     LogFlowFunc(("pBackendData=%#p Uuid=%RTuuid\n", pBackendData, pUuid));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc;
@@ -1911,8 +1931,9 @@ static int vciSetUuid(void *pBackendData, PCRTUUID pUuid)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnGetModificationUuid */
-static int vciGetModificationUuid(void *pBackendData, PRTUUID pUuid)
+static DECLCALLBACK(int) vciGetModificationUuid(void *pBackendData, PRTUUID pUuid)
 {
+    RT_NOREF1(pUuid);
     LogFlowFunc(("pBackendData=%#p pUuid=%#p\n", pBackendData, pUuid));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc;
@@ -1929,8 +1950,9 @@ static int vciGetModificationUuid(void *pBackendData, PRTUUID pUuid)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnSetModificationUuid */
-static int vciSetModificationUuid(void *pBackendData, PCRTUUID pUuid)
+static DECLCALLBACK(int) vciSetModificationUuid(void *pBackendData, PCRTUUID pUuid)
 {
+    RT_NOREF1(pUuid);
     LogFlowFunc(("pBackendData=%#p Uuid=%RTuuid\n", pBackendData, pUuid));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc;
@@ -1952,42 +1974,13 @@ static int vciSetModificationUuid(void *pBackendData, PCRTUUID pUuid)
 }
 
 /** @copydoc VDCACHEBACKEND::pfnDump */
-static void vciDump(void *pBackendData)
+static DECLCALLBACK(void) vciDump(void *pBackendData)
 {
     NOREF(pBackendData);
 }
 
-/** @copydoc VDCACHEBACKEND::pfnAsyncRead */
-static int vciAsyncRead(void *pBackendData, uint64_t uOffset, size_t cbRead,
-                        PVDIOCTX pIoCtx, size_t *pcbActuallyRead)
-{
-    int rc = VERR_NOT_SUPPORTED;
-    PVCICACHE pCache = (PVCICACHE)pBackendData;
 
-    return rc;
-}
-
-/** @copydoc VDCACHEBACKEND::pfnAsyncWrite */
-static int vciAsyncWrite(void *pBackendData, uint64_t uOffset, size_t cbWrite,
-                         PVDIOCTX pIoCtx, size_t *pcbWriteProcess)
-{
-    int rc = VERR_NOT_SUPPORTED;
-    PVCICACHE pCache = (PVCICACHE)pBackendData;
-
-    return rc;
-}
-
-/** @copydoc VDCACHEBACKEND::pfnAsyncFlush */
-static int vciAsyncFlush(void *pBackendData, PVDIOCTX pIoCtx)
-{
-    int rc = VERR_NOT_SUPPORTED;
-    PVCICACHE pCache = (PVCICACHE)pBackendData;
-
-    return rc;
-}
-
-
-VDCACHEBACKEND g_VciCacheBackend =
+const VDCACHEBACKEND g_VciCacheBackend =
 {
     /* pszBackendName */
     "vci",
@@ -1999,8 +1992,6 @@ VDCACHEBACKEND g_VciCacheBackend =
     s_apszVciFileExtensions,
     /* paConfigInfo */
     NULL,
-    /* hPlugin */
-    NIL_RTLDRMOD,
     /* pfnProbe */
     vciProbe,
     /* pfnOpen */
@@ -2015,6 +2006,8 @@ VDCACHEBACKEND g_VciCacheBackend =
     vciWrite,
     /* pfnFlush */
     vciFlush,
+    /* pfnDiscard */
+    NULL,
     /* pfnGetVersion */
     vciGetVersion,
     /* pfnGetSize */
@@ -2041,12 +2034,6 @@ VDCACHEBACKEND g_VciCacheBackend =
     vciSetModificationUuid,
     /* pfnDump */
     vciDump,
-    /* pfnAsyncRead */
-    vciAsyncRead,
-    /* pfnAsyncWrite */
-    vciAsyncWrite,
-    /* pfnAsyncFlush */
-    vciAsyncFlush,
     /* pfnComposeLocation */
     NULL,
     /* pfnComposeName */
