@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,9 +15,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <VBox/vd.h>
 #include <VBox/err.h>
 #include <VBox/log.h>
@@ -36,24 +37,26 @@
 #define VDI_TEST
 #define VMDK_TEST
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** The error count. */
 unsigned g_cErrors = 0;
 
 
-static void tstVDError(void *pvUser, int rc, RT_SRC_POS_DECL,
-                       const char *pszFormat, va_list va)
+static DECLCALLBACK(void) tstVDError(void *pvUser, int rc, RT_SRC_POS_DECL, const char *pszFormat, va_list va)
 {
+    RT_NOREF1(pvUser);
     g_cErrors++;
     RTPrintf("tstVD: Error %Rrc at %s:%u (%s): ", rc, RT_SRC_POS_ARGS);
     RTPrintfV(pszFormat, va);
     RTPrintf("\n");
 }
 
-static int tstVDMessage(void *pvUser, const char *pszFormat, va_list va)
+static DECLCALLBACK(int) tstVDMessage(void *pvUser, const char *pszFormat, va_list va)
 {
+    RT_NOREF1(pvUser);
     RTPrintf("tstVD: ");
     RTPrintfV(pszFormat, va);
     return VINF_SUCCESS;
@@ -118,9 +121,7 @@ static int tstVDCreateDelete(const char *pszBackend, const char *pszFilename,
 static int tstVDOpenDelete(const char *pszBackend, const char *pszFilename)
 {
     int rc;
-    PVBOXHDD pVD = NULL;
-    VDGEOMETRY       PCHS = { 0, 0, 0 };
-    VDGEOMETRY       LCHS = { 0, 0, 0 };
+    PVBOXHDD         pVD = NULL;
     PVDINTERFACE     pVDIfs = NULL;
     VDINTERFACEERROR VDIfError;
 
@@ -402,6 +403,8 @@ static void generateRandomSegments(PRNDCTX pCtx, PSEGMENT pSegment, uint32_t nSe
 
 static void mergeSegments(PSEGMENT pBaseSegment, PSEGMENT pDiffSegment, PSEGMENT pMergeSegment, uint32_t u32MaxLength)
 {
+    RT_NOREF1(u32MaxLength);
+
     while (pBaseSegment->u32Length > 0 || pDiffSegment->u32Length > 0)
     {
         if (pBaseSegment->u64Offset < pDiffSegment->u64Offset)
